#include "../include/aprecv-general.h"

int print_cdp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size, unsigned short length) {
  static struct cdp_pkt_header *cdp_header;
  unsigned short type, tot_length, cap, i=0;
  unsigned char *cdp_buf;
  
  cdp_header = (struct cdp_pkt_header *)(packet+data_link_size+opt_size);
  
  apsr_output("\n- CDP Header -\n  Version: %d",(int) cdp_header->version);
  apsr_output(", TTL: %ds\n",(int) cdp_header->ttl);
  apsr_output("  Checksum: 0x%04x\n", apsr_ntohs(cdp_header->sum));
  cdp_buf = (packet+data_link_size+opt_size+sizeof(cdp_header));
  while(i < length) {
    type        = (cdp_buf[i] <<  8) + cdp_buf[i+1];
    tot_length  = (cdp_buf[i+2] <<  8) + cdp_buf[i+3];

    if(i + tot_length > header->len) {
	    if(statistic) 
		    stats.tr.cdp++;
            goto end_cdp;
    }
    if(tot_length == 0)
	    break;
    apsr_output("  Type: %d", type);
    if(verbose) {
	    switch(type) {
		  case APSR_TYPE_CDP_DEVID:
   		    apsr_output(" (Device ID)");
		    break;
	          case APSR_TYPE_CDP_ADDR:
		    apsr_output(" (Address)");
		    break;
		  case APSR_TYPE_CDP_PORTID:
		    apsr_output(" (Port ID)");
		    break;
		  case APSR_TYPE_CDP_CAP:
		    apsr_output(" (Capabilities)");
		    break;
		  case APSR_TYPE_CDP_VER:
		    apsr_output(" (Version)");
		    break;
		  case APSR_TYPE_CDP_PLA:
		    apsr_output(" (Platform)");
		    break;
		  case APSR_TYPE_CDP_IPPRE:
		    apsr_output(" (IP Prefix)");
		    break;
                  case APSR_TYPE_CDP_HELLO:
                    apsr_output(" (Protocol-Hello)");
                    break;
                  case APSR_TYPE_CDP_VTP:
                    apsr_output(" (VTP Management Domain)");
                    break;
                  case APSR_TYPE_CDP_NAT_VLAN:
                    apsr_output(" (Native VLAN)");
                    break;
                  case APSR_TYPE_CDP_DUPLEX:
                    apsr_output(" (Duplex)");
                    break;
                  case APSR_TYPE_CDP_ATA_REQ:
                    apsr_output(" (ATA-186 VoIP VLAN request)");
                    break;
                  case APSR_TYPE_CDP_ATA_ASS:
                    apsr_output(" (ATA-186 VoIP VLAN assignment)");
                    break;
		  case APSR_TYPE_CDP_POWER:
                    apsr_output(" (Power consumption");
                    break;
                  case APSR_TYPE_CDP_MTU:
                    apsr_output(" (MTU)");
                    break;
                  case APSR_TYPE_CDP_AVVID_TR:
                    apsr_output(" (AVVID trusted bitmap)");
                    break;
                  case APSR_TYPE_CDP_AVVID_UT:
                    apsr_output(" (AVVID untrusted bitmap)");
                    break;
                  case APSR_TYPE_CDP_SYSNAME:
                    apsr_output(" (sysName)");
                    break;
                  case APSR_TYPE_CDP_SYSID:
                    apsr_output(" (sysObjectID)");
                    break;
                  case APSR_TYPE_CDP_MAN_ADDR:
                    apsr_output(" (Management addresses)");
                    break;
                  case APSR_TYPE_CDP_LOCAT:
                    apsr_output(" (Location)");
                    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
	    }
    }
    switch(type) {
	    case APSR_TYPE_CDP_DEVID:
              apsr_output("\n  Device ID: \"%.*s\"",
			      tot_length-4, cdp_buf+i+4);
              break;
            case APSR_TYPE_CDP_ADDR:
              /* TODO */
	      break;
	    case APSR_TYPE_CDP_PORTID:
              apsr_output("\n  Port ID: \"%.*s\"",
			      tot_length-4, cdp_buf+i+4);
              break;
            case APSR_TYPE_CDP_CAP:
	      cap = (unsigned) cdp_buf[i + 7];
	      apsr_output("\n  Capabilities: 0x%02x", cap);
	      if(verbose) {
		      if(cap & APSR_CAP_LEVEL3_ROUT) 
			apsr_output("\n  - 0x01 is set(Performs level 3 routing)");
	              else 
			apsr_output("\n  - 0x01 isn't set(Doesn't perform level 3 routing)");
		      if(cap & APSR_CAP_LEVEL2_TRANSBR)
			apsr_output("\n  - 0x02 is set(Performs level 2 transparent bridging)");
		      else
			apsr_output("\n  - 0x02 isn't set(Doesn't perform level 2 transparent bridging)");
		      if(cap & APSR_CAP_LEVEL2_SOURCBR)
		        apsr_output("\n  - 0x04 is set(Performs level 2 source-route bridging)");
		      else
			apsr_output("\n  - 0x04 isn't set(Doesn't perform level 2 source-route bridging)");
		      if(cap & APSR_CAP_LEVEL2_SWITCH)
			apsr_output("\n  - 0x08 is set(Performs level 2 switching)");
		      else
			apsr_output("\n  - 0x08 isn't set(Doesn't perform level 2 switching)");
		      if(cap & APSR_CAP_SEND_RECV_ONE)
		        apsr_output("\n  - 0x10 is set(Sends and receives packets for network-layer protocols)");
		      else
			apsr_output("\n  - 0x10 isn't set(Doesn't send or receive packets for network-layer protocols)");
		      if(cap & APSR_CAP_NOTFOR_IGMP) 
			apsr_output("\n  - 0x20 is set(Doesn't forward IGMP Report packets on nonrouter ports)");
		      else
			apsr_output("\n  - 0x20 isn't set(Forwards IGMP Report packets on nonrouter ports)");
		      if(cap & APSR_CAP_LEVEL1)
			apsr_output("\n  - 0x40 is set(Provides level 1 functionality)");
		      else
			apsr_output("\n  - 0x40 isn't set(Doesn't provide level 1 functionality)");
	      }
              break;
	    case APSR_TYPE_CDP_VER:
	      apsr_output("\n  Version: \"%.*s\"",
			      tot_length-4, cdp_buf+i+4);
              break;
            case APSR_TYPE_CDP_PLA:
	      apsr_output("\n  Platform: \"%.*s\"",
	                      tot_length-4, cdp_buf+i+4);
              break;
            case APSR_TYPE_CDP_IPPRE:
              apsr_output("\n  IPv4 Prefixes(%d):", tot_length / 5);
              if(tot_length % 5) {
		      if(statistic)
			      stats.tr.cdp++;
		      goto end_cdp;
              }				  
	      while(tot_length > 0) {
		      apsr_output(" %u.%u.%u.%u/%u", cdp_buf[i+4], cdp_buf[i+5],
				      cdp_buf[i+6], cdp_buf[i+7], cdp_buf[i+8]);
		      tot_length -= 5;
		      i += 5;
	      }
	      break;
            case APSR_TYPE_CDP_VTP:
              apsr_output("\n  VTP Management Domain: \"%.*s\"",
		                                tot_length-4, cdp_buf+i+4);
              break;
 	    case APSR_TYPE_CDP_NAT_VLAN:
	      apsr_output("\n  Native VLAN ID: %d", 
			      (cdp_buf[i + 4] << 8) + cdp_buf[i + 4 + 1]);
	      break;
	    case APSR_TYPE_CDP_DUPLEX:
	      apsr_output("\n  Duplex Mode: %s", 
			      cdp_buf[i + 4] ? "full" : "half");
	      break;
	    default:
	      break;
    }
    apsr_output("\n  Total length: %d\n", tot_length); 
    i += tot_length;
  }

end_cdp:
  
  if(statistic)
	stats.c.cdp++;
  
  return(RET_SUCCESS);
}
