#include "../include/aprecv-general.h"

static const char *igrpopcodes[]= {
	" (Unknown)",
	" (Update)",
	" (Request)",
};

int print_igrp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct igrp_pkt_header *igrp_header;
  static unsigned short version, op_code, pkt_checksum, correct_checksum;

  igrp_header = (struct igrp_pkt_header *)(packet+data_link_size+opt_size);
  
  version = APSR_IGRP_VERSION(igrp_header);
  op_code = APSR_IGRP_OP_CODE(igrp_header);

  apsr_output("- IGRP Header -\n  Version: %d, Opcode: %d",(int) version,(int) op_code);
  if(verbose)
	apsr_output("%s", igrpopcodes[op_code]);

  apsr_output("\n  Edition: %d\n",(int) igrp_header->edition);

  apsr_output("  Autosys: %d, Interior: %d\n", apsr_ntohs(igrp_header->autosys), 
		  apsr_ntohs(igrp_header->interior));
  apsr_output("  System: %d, Exterior: %d", apsr_ntohs(igrp_header->system),
		  apsr_ntohs(igrp_header->exterior));

  pkt_checksum = apsr_ntohs(igrp_header->sum);
  apsr_output("\n  Checksum: 0x%04x", pkt_checksum);

  if(calc_checksums) {
    igrp_header->sum=0x0000;
    correct_checksum=apsr_ntohs(in_cksum((unsigned short *)igrp_header, 
	sizeof(struct igrp_pkt_header)+(header->len-data_link_size-opt_size),
       	0));
    if(pkt_checksum == correct_checksum) {
	    apsr_output(" (correct checksum)");
    } 
    else {
	    apsr_output(" (incorrect checksum - should be 0x%04x)", 
			    correct_checksum);

            if(statistic)
		    stats.cr.igrp++;
    }
  }

  apsr_output("\n");
  
  if(statistic) 
          stats.c.igrp++;
  
  return(RET_SUCCESS);
}
