#include "../include/aprecv-general.h"

int print_ipv6_packet(struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct ipv6_pkt_header *ipv6_header;
  static unsigned short x, i=0;
  static unsigned short version, priority;

  ipv6_header = (struct ipv6_pkt_header *)(packet+data_link_size+opt_size);
  
  version = IPV6_VERSION(ipv6_header);
  priority = IPV6_PRIORITY(ipv6_header);
  
  apsr_output("\n- IPv6 Header -\n  Version: %d, Priority: %d\n",(int) version,(int) priority);
  apsr_output("  Flow Label: %d, Payload length: %d\n", apsr_ntohs(ipv6_header->flow_label), apsr_ntohs(ipv6_header->payload_len));
  apsr_output("  Next header: %d",(int) ipv6_header->next_header);
 
  if(verbose) 
	  print_ipv6_proto(ipv6_header->next_header);

  apsr_output(", Hop Limit: %d\n",(int) ipv6_header->hop_limit);

  apsr_output("  Src IP: ");
  for(x = 0; x <= 15; x++) {
  	if(((i++ % 2) == 0) && x != 0) 
		apsr_output(":");
	apsr_output("%02x", ipv6_header->src_ip.s6_addr[x]);
  }
  
  /* Prefix check Src IP */
  if(verbose) {
  	if(IN6_IS_ADDR_UNSPECIFIED(&ipv6_header->src_ip) == 0)
		apsr_output(" (unspecified)");
        else if(IN6_IS_ADDR_LOOPBACK(&ipv6_header->src_ip) == 0)
		apsr_output(" (loopback)");
        else if(IN6_IS_ADDR_MULTICAST(&ipv6_header->src_ip) == 0)
		apsr_output(" (multicast)");
	else if(IN6_IS_ADDR_LINKLOCAL(&ipv6_header->src_ip) == 0)
		apsr_output(" (linklocal)");
	else if(IN6_IS_ADDR_SITELOCAL(&ipv6_header->src_ip) == 0)
		apsr_output(" (sitelocal)");
	else if(IN6_IS_ADDR_V4MAPPED(&ipv6_header->src_ip) == 0)
		apsr_output(" (IPv4 mapped)");
	else if(IN6_IS_ADDR_V4COMPAT(&ipv6_header->src_ip) == 0)
		apsr_output(" (IPv4 compat.)");
  }

  apsr_output("\n  Dst IP: ");
  for(x = 0; x <= 15; x++) {
	if(((i++ % 2) == 0) && x != 0) 
		apsr_output(":");
	apsr_output("%02x", ipv6_header->dst_ip.s6_addr[x]);
  }
 
  /* Prefix check Dst IP */
   if(verbose) {
	   if(IN6_ARE_ADDR_EQUAL(&ipv6_header->src_ip, &ipv6_header->dst_ip) == 0) {
		   apsr_output(" (same as Src IP)");
	   }
	   else {
		   if(IN6_IS_ADDR_UNSPECIFIED(&ipv6_header->dst_ip) == 0)
			   apsr_output(" (unspecified)");
	           else if(IN6_IS_ADDR_LOOPBACK(&ipv6_header->dst_ip) == 0)
	                   apsr_output(" (loopback)");
        	   else if(IN6_IS_ADDR_MULTICAST(&ipv6_header->dst_ip) == 0)
	                   apsr_output(" (multicast)");
	       	   else if(IN6_IS_ADDR_LINKLOCAL(&ipv6_header->dst_ip) == 0)
        	           apsr_output(" (linklocal)");
	           else if(IN6_IS_ADDR_SITELOCAL(&ipv6_header->dst_ip) == 0)
        	           apsr_output(" (sitelocal)");
	           else if(IN6_IS_ADDR_V4MAPPED(&ipv6_header->dst_ip) == 0)
        	           apsr_output(" (IPv4 mapped)");
	           else if(IN6_IS_ADDR_V4COMPAT(&ipv6_header->dst_ip) == 0)
	                   apsr_output(" (IPv4 compat.)");
	 } 		  
  }
   
  apsr_output("\n");

  if(statistic)
	  stats.c.ipv6++;
 
  switchto_ipv6_proto(header, packet, opt_size+40, ipv6_header->next_header);
  
  return(RET_SUCCESS);
}

int print_ipv6_hh_packet(struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct ipv6_hh_pkt_header *ipv6_hh_header;

  ipv6_hh_header = (struct ipv6_hh_pkt_header *)(packet+data_link_size+opt_size);

  apsr_output("- IPv6 Hop-by-Hop Header -\n  Next header: %u",(int) ipv6_hh_header->next_header);
  
  if(verbose) 
	  print_ipv6_proto(ipv6_hh_header->next_header);

  apsr_output("\n  Header Extension Length: %u\n", ipv6_hh_header->length);

  if(statistic)
          stats.c.ipv6_hh++;

  switchto_ipv6_proto(header, packet, opt_size+2+ipv6_hh_header->length, ipv6_hh_header->next_header);

  return(RET_SUCCESS);
}

int print_ipv6_rt_packet(struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct ipv6_rt_pkt_header *ipv6_rt_header;
  
  ipv6_rt_header = (struct ipv6_rt_pkt_header *)(packet+data_link_size+opt_size);

  apsr_output("- IPv6 Routing Header -\n  Next header: %u",(int) ipv6_rt_header->next_header);

  if(verbose)
	  print_ipv6_proto(ipv6_rt_header->next_header);

  apsr_output("\n  Header Extension Length: %u, Routing Type: %u\n",
		  ipv6_rt_header->length, ipv6_rt_header->r_type);
  apsr_output("  Segments Left: %u\n", ipv6_rt_header->seg_left);
   
  if(statistic)
	  stats.c.ipv6_rt++;

  switchto_ipv6_proto(header, packet, opt_size+ipv6_rt_header->length, ipv6_rt_header->next_header);

  return(RET_SUCCESS);
}

void print_ipv6_proto(unsigned char protocol) {
  switch(protocol) {
          case IP6_NEXTHDR_HOP:
            apsr_output(" (hop-by-hop option header)");
            break;
          case IP6_NEXTHDR_TCP:
            apsr_output(" (TCP)");
            break;
          case IP6_NEXTHDR_EGP:
            apsr_output(" (EGP)");
            break;
          case IP6_NEXTHDR_COMP:
            apsr_output(" (IPComp)");
            break;
          case IP6_NEXTHDR_UDP:
            apsr_output(" (UDP)");
            break;
          case IP6_NEXTHDR_IP6:
            apsr_output(" (IPv6 in IPv6)");
            break;
          case IP6_NEXTHDR_ROUTING:
            apsr_output(" (Routing header)");
            break;
          case IP6_NEXTHDR_FRAGMENT:
            apsr_output(" (Fragmentation/reassembly header)");
            break;
          case IP6_NEXTHDR_ESP:
            apsr_output(" (Encapsulating security payload)");
            break;
          case IP6_NEXTHDR_AUTH:
            apsr_output(" (Authentication header)");
            break;
          case IP6_NEXTHDR_ICMP:
            apsr_output(" (ICMP)");
            break;
          case IP6_NEXTHDR_NONE:
            apsr_output(" (No next header)");
            break;
          case IP6_NEXTHDR_DEST:
            apsr_output(" (Destination options header)");
            break;
          case IP6_NEXTHDR_PIM:
            apsr_output(" (PIM)");
            break;
          default:
            break;
  }
}
		  
void switchto_ipv6_proto(struct aprecv_frame_header *header, unsigned char *packet, int opt_size, unsigned char protocol) {
  switch(protocol) {	
        case IP6_NEXTHDR_HOP:
          print_ipv6_hh_packet(header, packet, opt_size);
	  break;
	case IP6_NEXTHDR_IP6:
          print_ipv6_packet(header, packet, opt_size);
          break;
        case IP6_NEXTHDR_TCP:
          print_tcp_packet(header, packet, opt_size, 6);
          break;
        case IP6_NEXTHDR_EGP:
          print_egp_packet(header, packet, opt_size);
          break;
        case IP6_NEXTHDR_UDP:
          print_udp_packet(header, packet, opt_size, 6);
          break;
	case IP6_NEXTHDR_ROUTING:
	  print_ipv6_rt_packet(header, packet, opt_size);
	  break;
        case IP6_NEXTHDR_ICMP:
          print_icmpv6_packet(packet, opt_size);
          break;
        case IP6_NEXTHDR_AUTH:
          print_ipah_packet(header, packet, opt_size);
          break;
        case IP6_NEXTHDR_ESP:
          print_ipesp_packet(packet, opt_size);
          break;
        case IP6_NEXTHDR_COMP:
          print_ipcomp_packet(packet, opt_size);
          break;
        case IP6_NEXTHDR_PIM:
          print_pim_packet(packet, opt_size);
          break;
        default:
          break;
  }
}  
