#include "../include/aprecv-general.h"

static const char *narptype[] = {
	" (Reserved)",
	" (Request)",
	" (Reply)",
};

static const char *narpcode[] = {
 	" (Reserved)",
	" (NARP Request)",
	" (NARP Request for Authoritative Information)",
};

static const char *narp_posneg_code[] = {
	" (Reserved)",
	" (Positive non-authoritative reply)",
	" (Positive authoritative reply)",
	" (Negative non-authoritative reply)",
	" (Negative authoritative reply)",
};

int print_narp_packet (unsigned char *packet, int opt_size) {
  static struct narp_pkt_header *narp_header;
  static char *source_ip, *dest_ip;
  
  narp_header = (struct narp_pkt_header *)(packet+data_link_size+opt_size);
  source_ip   = strdup(inet_ntoa(narp_header->src_ip));
  dest_ip     = strdup(inet_ntoa(narp_header->dst_ip));
  
  apsr_output("- NARP Header -\n  Version: %d\n",(int) narp_header->version);
  apsr_output("  Hop Count: %d, Checksum: 0x%04x\n",(int) narp_header->hop, 
		  apsr_ntohs(narp_header->sum));
  apsr_output("  Type: %d",(int) narp_header->type);
  
  if(verbose) 
	  apsr_output("%s", narptype[narp_header->type]);
		  
  apsr_output("\n  Code: %d",(int) narp_header->code);
  if(verbose) {
          switch(narp_header->type) {
		  case APSR_NARP_TYPE_REQUEST:
		    apsr_output("%s", narpcode[narp_header->code]);
		    break;
		  case APSR_NARP_TYPE_REPLY:
		    apsr_output("%s", narp_posneg_code[narp_header->code]);
		    break;
                  default:
                    break;
          }
  }	  
  apsr_output("\n  Src IP: %s", source_ip);
  if(verbose) {
/*        resolve_ip(source_ip, AF_INET); */
  }
  apsr_output("\n  Dst IP: %s", dest_ip);
  if(verbose) {
/*        resolve_ip(dest_ip, AF_INET); */
  }
  apsr_output("  Length: %d\n",(int) narp_header->length);                
	
  if(statistic)
	stats.c.narp++;

  free(source_ip);
  free(dest_ip);

  return(RET_SUCCESS);
}
