#include "../include/aprecv-general.h"

static const char *pimv1type[]= {
	" (Query)",
	" (Register)",
	" (Register-Stop)",
	" (Join/Prune)",
	" (RP-reachable)",
	" (Assert)",
	" (Graft)",
	" (Graft-Ack)",
};

static const char *pimv2type[]= {
	" (Hello)",
	" (Register)",
	" (Register-Stop)",
	" (Join/Prune)",
	" (Bootstrap)",
	" (Assert)",
	" (Graft)",
	" (Graft-Ack)",
	" (Candidate-RP-Advertisement)",
	" (Graft-Ack)",
};	
	
int print_pim_packet (unsigned char *packet, int opt_size) {
  static struct pim_pkt_header *pim_header;
  static unsigned short version, type, pkt_checksum, correct_checksum;
/*  static unsigned short option_type, option_length, option_value_short; */
  
  pim_header = (struct pim_pkt_header *)(packet+data_link_size+opt_size);
  
  version 	= APSR_PIM_VERSION(pim_header);
  type		= APSR_PIM_TYPE(pim_header);

  apsr_output("- PIM Header -\n  Version: %d\n  Type: %d", version, type);
 
  if(verbose && version == 1) 
	  apsr_output("%s", pimv1type[type]);
  else if(verbose && version == 2) 
	  apsr_output("%s", pimv2type[type]);
		 
  apsr_output("\n  Reserved: %d\n", pim_header->reserved);

  /* PIM over IPv4 - checksum works, but PIM over IPv6 - needs pseudo header for
   * checksum calculation */
  
  pkt_checksum = apsr_ntohs(pim_header->sum);
  apsr_output("  Checksum: 0x%04x", pkt_checksum);

  if(calc_checksums) {
    pim_header->sum=0x0000;
    correct_checksum=apsr_ntohs(in_cksum((unsigned short *)pim_header, 4, 0));
    if(pkt_checksum == correct_checksum) {
	    apsr_output(" (correct checksum)");
    } 
    else {
	    apsr_output(" (incorrect checksum - should be 0x%04x)", 
			    correct_checksum);

 	    if(statistic && version == 1)
	    	stats.cr.pimv1++;
	    else if(statistic && version == 2)
	    	stats.cr.pimv2++; 
    }
  }
  apsr_output("\n");

  if(version == 1) {
    if(statistic) 
    	stats.c.pimv1++;
  }
  else if(version == 2) {
    if(statistic) 
    	stats.c.pimv2++;
/*    switch(type) {
   	case APSR_PIMV2_TYPE_HELLO:
          option_type   = (packet+data_link_size+opt_size+4);
	  option_type   = apsr_ntohs(option_type);
	  option_length = (packet+data_link_size+opt_size+6);
	  option_length = apsr_ntohs(option_length);
	  apsr_output("  - Option(%u) -\n    Type: %u", option_type);
	  if(verbose) {
	    switch(option_type) {
	    	  case APSR_PIMV2_TYPE_HELLO_HOLDTIME:
		    apsr_output(" (Hold-time)");
		    break;
		  default:
		    apsr_output(" (Unknown)");
		    break;
            }  
	  }
	  apsr_output("\n    Length: %u", option_length);
	  switch(option_type) {
	  	case APSR_PIMV2_TYPE_HELLO_HOLDTIME:
	          option_value_short = (packet+data_link_size+opt_size+8); 
		  option_value_short = apsr_ntohs(option_value_short);
		  apsr_output("  Hold-time: %usec", option_value_short);
	          break;
		default:
		  break;
          }
          break;
	default:
	  break;
    }							       
*/  }
 
  return(RET_SUCCESS);
}
