#include "../include/aprecv-general.h"

/* this is for all common lcp's */
static const char *std_codes[] = { 
        " (Vendor Specific)",
	" (Configuration-Request)",
	" (Configuration-Ack)",
	" (Configuration-Nak)",
	" (Configuration-Reject)",
	" (Termination-Request)",
	" (Termination-Reply)",
	" (Code-Reject)",
};								

/* Same as above but with extensions for reset-request/reply */
static const char *ext_codes[] = {
	" (Vendor Specific)",
	" (Configuration-Request)",
	" (Configuration-Ack)",
	" (Configuration-Nak)",
	" (Configuration-Reject)",
	" (Termination-Request)",
	" (Termination-Reply)",
	" (Code-Reject)",
	" (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
        " (Unknown)",
	" (Reset-Request)",
	" (Reset-Reply)",	
};

static const char *lcpcodes[] = {
	" (Vendor Specific)",
	" (Configuration-Request)",
	" (Configuration-Ack)",
	" (Configuration-Nak)",
	" (Configuration-Reject)",
	" (Termination-Request)",
	" (Termination-Reply)",
	" (Code-Reject)",
	" (Protocol-Reject)",
	" (Echo-Request)",
	" (Echo-Reply)",
	" (Discard-Request)",
	" (Identification)",
	" (Time-Remaining)",
};

static const char *lcpconftype[] = {
	" (Reserved)",
	" (Maximum-Receive-Unit)",
	" (Async-Control-Character-Map)",
	" (Authentication-Protocol)",
	" (Quality-Protocol)",
	" (Magic-Number)",
	" (Reserved)",
	" (Protocol-Field-Compression)",
	" (Address-and-Control-Field-Compression)",
	" (FCS-Alternatives)",
	" (Self-Describing-Pad)",
	" (Numbered-Mode)",
	" (Multi-Link-Procedure)",
	" (Callback)",
	" (Connect-Time)",
	" (Compound-Frames)",
	" (Nominal-Data-Encapsulation)",
	" (Multilink-MRRU)",
	" (Multilink-Short-Sequence-Number-Header-Format)",
	" (Multilink-Endpoint-Discriminator)",
	" (Proprietary)",
	" (DCE-Identifier)",
	" (MP+ Procedure)",
	" (Link Discriminator for BACP)",
	" (LCP-Authentication)",
	" (COBS)",
	" (variable Prefix Elision)",
	" (Multilink header format)",
	" (Internationalization)",
	" (Simple Data Link on SONET/SDH)",
};		



int print_ppp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct ppp_pkt_header *ppp_header;
  static unsigned short protocol;

  ppp_header = (struct ppp_pkt_header *)(packet+data_link_size+opt_size);

  protocol = apsr_ntohs(ppp_header->protocol);
  
  apsr_output("- PPP Header -\n  Protocol: 0x%x", protocol);
  
  switch(protocol) {
          case APSR_PPP_PROTO_IP:
            if(verbose)
    		    apsr_output(" (IPv4)");
	    /* 2 octets for PPP header */
	    print_ipv4_packet(header, packet, opt_size+2);
	    break;
	  case APSR_PPP_PROTO_APPLE_TALK:
	    if(verbose) 
		    apsr_output(" (AppleTalk)");
	    break;
	  case APSR_PPP_PROTO_IPX:
	    if(verbose) 
		    apsr_output(" (IPX)");
	    /* 2 octets for PPP header */
	    print_ipx_packet(packet, opt_size+2);
	  case APSR_PPP_PROTO_IP6:
	    if(verbose)
		    apsr_output(" (IPv6)");
	    /* 2 octets for PPP header */
	    print_ipv6_packet(header, packet, opt_size+2);
	    break;
          case APSR_PPP_PROTO_COMPR_TCP:
	    if(verbose) 
		    apsr_output(" (Compressed TCP)");
	    break;
	  case APSR_PPP_PROTO_UNCOM_TCP:
	    if(verbose)
		    apsr_output(" (Uncompressed TCP)");
	    break;
	  case APSR_PPP_PROTO_SNALLC:
	    if(verbose)
		    apsr_output(" (SNA over LLC)");
	    /* 2 octets for PPP header */
	    print_llc_packet(header, packet, opt_size+2, header->caplen-opt_size-2);
	    break;
	  case APSR_PPP_PROTO_SNALLCCP:
          case APSR_PPP_PROTO_SNACP:
            if(verbose)
		    apsr_output(" (SNACP)");
            /* 2 octets for PPP header */
            print_ppp_snacp(packet, opt_size+2);
            break;
	  case APSR_PPP_PROTO_IPCP:
	    if(verbose)
		    apsr_output(" (IPv4 Control Protocol)");
	    /* 2 octets for PPP header */
	    print_ppp_ipcp(packet, opt_size+2);
	    break;
          case APSR_PPP_PROTO_ATCP:
            if(verbose)
                    apsr_output(" (AppleTalk Control Protocol)");
            /* 2 octets for PPP header */
            print_ppp_atcp(packet, opt_size+2);
            break;
	  case APSR_PPP_PROTO_IPXCP:
            if(verbose)
                    apsr_output(" (IPX Control Protocol)");
            /* 2 octets for PPP header */
            print_ppp_ipxcp(packet, opt_size+2);
            break;
          case APSR_PPP_PROTO_ECP:
            if(verbose)
                    apsr_output(" (Encryption Control Protocol)");
            /* 2 octets for PPP header */
            print_ppp_ecp(packet, opt_size+2);
            break;
	  case APSR_PPP_PROTO_LCP:
	    if(verbose)
		    apsr_output(" (LCP)");
	    /* 2 octets for PPP header */
	    print_ppp_lcp(packet, opt_size+2);
	    break;
          case APSR_PPP_PROTO_PAP:
	    if(verbose)
		    apsr_output(" (PAP)");
	    /* 2 octets for PPP header */
	    print_ppp_pap(packet, opt_size+2);
	    break;
	  case APSR_PPP_PROTO_LQR:
	    if(verbose)
		    apsr_output(" (Link Quality Report)");
            /* 2 octets for PPP header */
            print_ppp_lqr(packet, opt_size+2);
	    break;
	  case APSR_PPP_PROTO_CHA:
	    if(verbose)
		    apsr_output(" (CHAP)");
            /* 2 octets for PPP header */
            print_ppp_chap(packet, opt_size+2);
            break;
          case APSR_PPP_PROTO_EAP:
            if(verbose)
                    apsr_output(" (EAP)");
            /* 2 octets for PPP header */
            print_ppp_eap(packet, opt_size+2);
            break;
          case APSR_PPP_PROTO_BCP:
            if(verbose)
                    apsr_output(" (BCP)");
	    print_ppp_bcp(packet, opt_size+2);
            break;
          case APSR_PPP_PROTO_BAP:
            if(verbose) 
                    apsr_output(" (BAP)");
            break;
          case APSR_PPP_PROTO_CCP:
            if(verbose)
                    apsr_output(" (CCP)");
            /* 2 octets for PPP header */
            print_ppp_ccp(packet, opt_size+2);
	    break;
          case APSR_PPP_PROTO_LEX:
            if(verbose)
                    apsr_output(" (LEX)");
            break;
          case APSR_PPP_PROTO_DECNET:
            if(verbose)
                    apsr_output(" (Decnet Phase IV)");
            break;
          case APSR_PPP_PROTO_DECNETCP:
            if(verbose)
                    apsr_output(" (Decnet Phase IV CP)");
	    /* 2 octets for PPP header */
	    print_ppp_decnetcp(packet, opt_size+2);
            break;
          case APSR_PPP_PROTO_OSI:
            if(verbose)
                    apsr_output(" (OSI Network Layer)");
            break;
          case APSR_PPP_PROTO_OSICP:
            if(verbose)
                    apsr_output(" (OSI Network Layer CP)");
            /* 2 octets for PPP header */
            print_ppp_osinlcp(packet, opt_size+2);
            break;
	  case APSR_PPP_PROTO_SPCP:
            if(verbose)
                    apsr_output(" (Stream Protocol CP)");
            break;
          case APSR_PPP_PROTO_VINESCP:
            if(verbose)
                    apsr_output(" (Banyan Vines CP)");
            break;
	  case APSR_PPP_PROTO_IP6CP:
            if(verbose)
                    apsr_output(" (IPv6 Control Protocol)");
	    /* 2 octets for PPP header */
            print_ppp_ipv6cp(packet, opt_size+2);
	    break;
	  default:
	    if(verbose) {
                    if(protocol >= 0x0001 && protocol <= 0x001f) 
                            apsr_output(" (unknown reserved)");
                    else if(protocol >= 0x0020 && protocol <= 0x3fff) 
                            apsr_output(" (unknown network layer)");
                    else if(protocol >= 0x4000 && protocol <= 0x7fff)
                            apsr_output(" (unknown free/low bandwidth)");
                    else if(protocol >= 0x8000 && protocol <= 0xbfff)
                            apsr_output(" (unknown NCP)");
                    else 
                            apsr_output(" (unknown LCP)");
	    }
            break; 
  }

  if(statistic) 
        stats.c.ppp++;

  return(RET_SUCCESS);
}

int print_ppp_pap (unsigned char *packet, int opt_size) {
  static struct ppp_pap_header *ppp_pap_header;
  static unsigned int auth_length, x;
  static unsigned char *peer_id_length, *passwd_length, *msg_length;
  static unsigned char *peer_id, *password, *message;

  ppp_pap_header = (struct ppp_pap_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - PAP Header -\n    Code: %d",(int) ppp_pap_header->code);
  if(verbose) {
	  switch(ppp_pap_header->code) {
		  case APSR_PPP_PAP_AUTH_REQ:
		    apsr_output(" (Authenticate-Request)");
		    break;
		  case APSR_PPP_PAP_AUTH_ACK:
		    apsr_output(" (Authenticate-Ack)");
		    break;
		  case APSR_PPP_PAP_AUTH_NAK:
		    apsr_output(" (Authenticate-Nak)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
	 }
  }
	        
  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_pap_header->identifier, apsr_ntohs(ppp_pap_header->length));

  switch(ppp_pap_header->code) {
	  case APSR_PPP_PAP_AUTH_REQ:
            auth_length = data_link_size+sizeof(struct ppp_pap_header)+opt_size;
	    peer_id_length = (packet+auth_length);
	    auth_length++;
	    peer_id = (packet+auth_length);
	    auth_length += peer_id_length[0];
	    passwd_length = (packet+auth_length);
	    auth_length++;
	    password = (packet+auth_length);
	    apsr_output("    Peer-ID Length: %d\n    Peer-ID: ", peer_id_length[0]);
	    for(x = 0; x < peer_id_length[0]; x++) {
	      if(isprint(peer_id[x]))
		      apsr_output("%c", peer_id[x]);
	      else 
		      apsr_output(".");
	    }
	    apsr_output("\n    Password Length: %d\n    Password: ", passwd_length[0]);
	    for(x = 0; x < passwd_length[0]; x++) {
	      if(isprint(password[x]))
		      apsr_output("%c", password[x]);
	      else
		      apsr_output(".");
	    }
	    apsr_output("\n");
            break;
	  case APSR_PPP_PAP_AUTH_ACK:
	  case APSR_PPP_PAP_AUTH_NAK:
	    auth_length = data_link_size+sizeof(struct ppp_pap_header)+opt_size;
	    msg_length = (packet+auth_length);
	    auth_length++;
	    message = (packet+auth_length);
	    apsr_output("    Message Length: %d\n    Message: ", msg_length[0]);
	    for(x = 0; x < msg_length[0]; x++) {
              if(isprint(message[x]))
		      apsr_output("%c", message[x]);
	      else 
		      apsr_output(".");
	    }
	    apsr_output("\n");
	    break;
	  default:
	    break;
  }	  

  if(statistic) 
	  stats.c.ppp_pap++;
	
  return(RET_SUCCESS);
}

int print_ppp_chap (unsigned char *packet, int opt_size) {
  static struct ppp_chap_header *ppp_chap_header;
  static unsigned char *value, *message;
  static unsigned short x, length;

  ppp_chap_header = (struct ppp_chap_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_chap_header->length);

  apsr_output("\n  - CHAP Header -\n    Code: %d",(int) ppp_chap_header->code);

  if(verbose) {
	  switch(ppp_chap_header->code) {
		  case APSR_PPP_CHAP_CHALLENGE:
		    apsr_output(" (Challenge)");
                    break;
		  case APSR_PPP_CHAP_RESPONSE:
		    apsr_output(" (Response)");
		    break;
		  case APSR_PPP_CHAP_SUCCESS:
		    apsr_output(" (Success)");
		    break;
		  case APSR_PPP_CHAP_FAILURE:
		    apsr_output(" (Failure)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
          }
  }			    
										    
  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_chap_header->identifier,(int) length); 

  switch(ppp_chap_header->code) {
	case APSR_PPP_CHAP_CHALLENGE:
        case APSR_PPP_CHAP_RESPONSE:
          value = (packet+data_link_size+sizeof(struct ppp_chap_header)+opt_size);
          apsr_output("    Value-Size: %d\n    Value: \"", value[0]);
          for(x = 0; x < value[0]; x++) {
            if(isprint(value[x])) 
		    apsr_output("%c", value[x]);
            else
                    apsr_output("."); 
          }
          apsr_output("\"\n    Name: \"");
          length = length-(sizeof(struct ppp_chap_header))-1-value[0]; 
          for(x = 0; x <= length; x++) {
            if(isprint(value[x+value[0]+1]))
                    apsr_output("%c", value[x+value[0]+1]);
            else
                    apsr_output(".");
          }   
          apsr_output("\"\n");
          break;  
        case APSR_PPP_CHAP_SUCCESS:
        case APSR_PPP_CHAP_FAILURE:
	  message = (packet+data_link_size+sizeof(struct ppp_chap_header)+opt_size);
	  apsr_output("    Message: \"");
	  length = length-(sizeof(struct ppp_chap_header))-1;
          for(x = (sizeof(struct ppp_chap_header)+1); x <= length; x++) {
            if(isprint(message[x]))
	    	    apsr_output("%c", message[x]);
	    else
	            apsr_output(".");
          }
          apsr_output("\"\n");
          break;
        default:
          break;
  }	

  if(statistic) 
	  stats.c.ppp_chap++;

  return(RET_SUCCESS);
}

int print_ppp_eap (unsigned char *packet, int opt_size) {
  static struct ppp_eap_header *ppp_eap_header;
  static unsigned short length;

  ppp_eap_header = (struct ppp_eap_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_eap_header->length);

  apsr_output("\n  - EAP Header -\n    Code: %d",(int) ppp_eap_header->code);

  if(verbose) {
	  switch(ppp_eap_header->code) {
		  case APSR_PPP_EAP_REQUEST:
		    apsr_output(" (Request)");
                    break;
		  case APSR_PPP_EAP_RESPONSE:
		    apsr_output(" (Response)");
		    break;
		  case APSR_PPP_EAP_SUCCESS:
		    apsr_output(" (Success)");
		    break;
		  case APSR_PPP_EAP_FAILURE:
		    apsr_output(" (Failure)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
          }
  }			    
										    
  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_eap_header->identifier,(int) length); 

  if(statistic) 
	  stats.c.ppp_eap++;

  return(RET_SUCCESS);
}

int print_ppp_bcp (unsigned char *packet, int opt_size) {
  static struct ppp_bcp_header *ppp_bcp_header;
  static unsigned short length;

  ppp_bcp_header = (struct ppp_bcp_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_bcp_header->length);

  apsr_output("\n  - BCP Header -\n    Code: %d",(int) ppp_bcp_header->code);

  if(verbose)
	  apsr_output("%s", std_codes[ppp_bcp_header->code]);
  
  apsr_output("\n    Identifier: %d, Length: %d\n",
		  (int) ppp_bcp_header->identifier,(int) length); 

  if(statistic) 
	  stats.c.ppp_bcp++;

  return(RET_SUCCESS);
}
	
int print_ppp_ipcp (unsigned char *packet, int opt_size) {
  static struct ppp_ipcp_header *ppp_ipcp_header;
  static struct dummy_ip *dummy_ip;
  static char *ip_addr;
	 
  ppp_ipcp_header = (struct ppp_ipcp_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - IPCP Header -\n    Code: %d",(int) ppp_ipcp_header->code);

  if(verbose) 
	  apsr_output("%s", std_codes[ppp_ipcp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_ipcp_header->identifier, apsr_ntohs(ppp_ipcp_header->length));
 
  switch(ppp_ipcp_header->code) {
	  case APSR_PPP_IPCP_CON_REQ:
	  case APSR_PPP_IPCP_CON_ACK:
	  case APSR_PPP_IPCP_CON_NAK:
	  case APSR_PPP_IPCP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d", 
			    (int)ppp_ipcp_header->ext.conf.type);
	    
	    if(verbose) {
		    switch(ppp_ipcp_header->ext.conf.type) {
			    case APSR_PPP_IPCP_TYPE_IPADDRS:
			      apsr_output(" (IP Addresses)");
			      break;
			    case APSR_PPP_IPCP_TYPE_IPCOMP:
			      apsr_output(" (IP-Compression-Protocol)");
			      break;
                            case APSR_PPP_IPCP_TYPE_IP:
                              apsr_output(" (IP Address)");
                              break;
                            case APSR_PPP_IPCP_TYPE_MOIP:
                              apsr_output(" (Mobile-IPv4)");
                              break;
                            case APSR_PPP_IPCP_TYPE_PRDNS:
                              apsr_output(" (Primary DNS Server Address)");
                              break;
                            case APSR_PPP_IPCP_TYPE_PRNBNS:
                              apsr_output(" (Primary NBNS Server Address)");
                              break;
                            case APSR_PPP_IPCP_TYPE_SEDNS:
                              apsr_output(" (Secondary DNS Server Address)");
                              break;
                            case APSR_PPP_IPCP_TYPE_SENBNS:
                              apsr_output(" (Secondary NBNS Server Address)");
                              break;
                            default:
                              apsr_output(" (unkown)");
                              break;
                    }
	    }
	    apsr_output("\n      Length: %d\n",(int) ppp_ipcp_header->ext.conf.length);
            if((ppp_ipcp_header->ext.conf.type == APSR_PPP_IPCP_TYPE_IP ||
		ppp_ipcp_header->ext.conf.type == APSR_PPP_IPCP_TYPE_MOIP ||
		ppp_ipcp_header->ext.conf.type == APSR_PPP_IPCP_TYPE_PRDNS ||
		ppp_ipcp_header->ext.conf.type == APSR_PPP_IPCP_TYPE_PRNBNS ||
		ppp_ipcp_header->ext.conf.type == APSR_PPP_IPCP_TYPE_SEDNS || 
		ppp_ipcp_header->ext.conf.type == APSR_PPP_IPCP_TYPE_SENBNS)) {
		  if(ppp_ipcp_header->ext.conf.length == 6) {
			  dummy_ip = (struct dummy_ip *)(packet+data_link_size+
				      opt_size+6);		  
			  ip_addr = strdup(inet_ntoa(dummy_ip->ip_address));
		          switch(ppp_ipcp_header->ext.conf.type) {
				  case APSR_PPP_IPCP_TYPE_IP:
		                    apsr_output("      IP-Address: %s\n", ip_addr);
				    break;
		                  case APSR_PPP_IPCP_TYPE_MOIP:
			            apsr_output("      Mobile-IPv4: %s\n", ip_addr);
			            break;
                                  case APSR_PPP_IPCP_TYPE_PRDNS:
                                    apsr_output("      Primary DNS: %s\n", ip_addr);
                                    break;
                                  case APSR_PPP_IPCP_TYPE_PRNBNS:
                                    apsr_output("      Primary NBNS: %s\n", ip_addr);
                                    break;
                                  case APSR_PPP_IPCP_TYPE_SEDNS:
                                    apsr_output("      Secondary DNS: %s\n", ip_addr);
                                    break;
                                  case APSR_PPP_IPCP_TYPE_SENBNS:
                                    apsr_output("      Secondary NBNS: %s\n", ip_addr);
                                    break;
		          }	
			  free(ip_addr);
                  }
		  else {
			  apsr_output("      Error: Invalid length in IPCP configuration header\n");
			  
			  if(statistic)
				  stats.err.ppp_ipcp++;
		  }
	    }
	    break;
	  default:
	    break;
  }	    
  
  if(statistic) 
	  stats.c.ppp_ipcp++;

  return(RET_SUCCESS);	
}
	
int print_ppp_ipv6cp (unsigned char *packet, int opt_size) {
  static struct ppp_ipv6cp_header *ppp_ipv6cp_header;
	 
  ppp_ipv6cp_header = (struct ppp_ipv6cp_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - IPV6CP Header -\n    Code: %d",(int) ppp_ipv6cp_header->code);

  if(verbose)
	  apsr_output("%s", std_codes[ppp_ipv6cp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_ipv6cp_header->identifier, apsr_ntohs(ppp_ipv6cp_header->length));
 
  switch(ppp_ipv6cp_header->code) {
	  case APSR_PPP_IPV6CP_CON_REQ:
	  case APSR_PPP_IPV6CP_CON_ACK:
	  case APSR_PPP_IPV6CP_CON_NAK:
	  case APSR_PPP_IPV6CP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d", 
			    (int)ppp_ipv6cp_header->ext.conf.type);
	    
	    if(verbose) {
		    switch(ppp_ipv6cp_header->ext.conf.type) {
			    case APSR_PPP_IPV6CP_TYPE_IDENT:
			      apsr_output(" (Interface-Identifier)");
			      break;
			    case APSR_PPP_IPV6CP_TYPE_IPCOMP:
			      apsr_output(" (IPv6-Compression-Protocol)");
			      break;
                            default:
                              apsr_output(" (unkown)");
                              break;
                    }
	    }
	    apsr_output("\n      Length: %d\n",(int) ppp_ipv6cp_header->ext.conf.length);
	    break;
	  default:
	    break;
  }	    
  
  if(statistic) 
	  stats.c.ppp_ipv6cp++;

  return(RET_SUCCESS);	
}

int print_ppp_ipxcp (unsigned char *packet, int opt_size) {
  static struct ppp_ipxcp_header *ppp_ipxcp_header;
  
  ppp_ipxcp_header = (struct ppp_ipxcp_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - IPXCP Header -\n    Code: %d",(int) ppp_ipxcp_header->code);

  if(verbose)
	  apsr_output("%s", std_codes[ppp_ipxcp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_ipxcp_header->identifier, apsr_ntohs(ppp_ipxcp_header->length));
 
  switch(ppp_ipxcp_header->code) {
          case APSR_PPP_IPXCP_CON_REQ:
          case APSR_PPP_IPXCP_CON_ACK:
          case APSR_PPP_IPXCP_CON_NAK:
          case APSR_PPP_IPXCP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d",
              (int)ppp_ipxcp_header->ext.conf.type);
            if(verbose) {
		  switch(ppp_ipxcp_header->ext.conf.type) {
			    case APSR_PPP_IPXCP_TYPE_NETNUM:
			      apsr_output(" (IPX-Network-Number)");
			      break;
			    case APSR_PPP_IPXCP_TYPE_NODNUM:
			      apsr_output(" (IPX-Node-Number)");
			      break;
			    case APSR_PPP_IPXCP_TYPE_IPXCOMP:
			      apsr_output(" (IPX-Compression-Protocol)");
			      break;
			    case APSR_PPP_IPXCP_TYPE_IPXROUT:
			      apsr_output(" (IPX-Routing-Protocol)");
			      break; 
			    case APSR_PPP_IPXCP_TYPE_ROUTNAM:
			      apsr_output(" (IPX-Router-Name)");
			      break;
			    case APSR_PPP_IPXCP_TYPE_CONFCOM:
			      apsr_output(" (IPX-Configuration-Complete)");
			      break;
			    default:
			      apsr_output(" (unkown)");
	                      break;
                  }
            }
            apsr_output("\n      Length: %d\n",(int) ppp_ipxcp_header->ext.conf.length);
            break;
          default:
            break;
  }

  if(statistic)
	  stats.c.ppp_ipxcp++;

  return(RET_SUCCESS);	
}

int print_ppp_atcp (unsigned char *packet, int opt_size) {
  static struct ppp_atcp_header *ppp_atcp_header;
  unsigned short routing_proto;
  
  ppp_atcp_header = (struct ppp_atcp_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - ATCP Header -\n    Code: %d",(int) ppp_atcp_header->code);

  if(verbose)
	  apsr_output("%s", std_codes[ppp_atcp_header->code]);
  
  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_atcp_header->identifier, apsr_ntohs(ppp_atcp_header->length));

  switch(ppp_atcp_header->code) {
	  case APSR_PPP_ATCP_CON_REQ:
	  case APSR_PPP_ATCP_CON_ACK:
	  case APSR_PPP_ATCP_CON_NAK:
	  case APSR_PPP_ATCP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d",
                            (int)ppp_atcp_header->ext.conf.type);
            if(verbose) {
		    switch(ppp_atcp_header->ext.conf.type) {
			    case APSR_PPP_ATCP_TYPE_APPLETADR:
                              apsr_output(" (AppleTalk Address)");
                              break;
                            case APSR_PPP_ATCP_TYPE_ROUTING:
                              apsr_output(" (Routing protocol)");
                              break;
                            case APSR_PPP_ATCP_TYPE_SUPBROAD:
                              apsr_output(" (Suppress Broadcast)");
                              break;
                            case APSR_PPP_ATCP_TYPE_ATCOMP:
                              apsr_output(" (AppleTalk Compression Protocol)");
                              break;
                            case APSR_PPP_ATCP_TYPE_RESERV:
                              apsr_output(" (Reserved)");
                              break;
                            case APSR_PPP_ATCP_TYPE_SERVER:
                              apsr_output(" (Server Information)");
                              break;
                            case APSR_PPP_ATCP_TYPE_ZONE:
                              apsr_output(" (Zone Information)");
                              break;
                            case APSR_PPP_ATCP_TYPE_DEFROUTER:
                              apsr_output(" (Default Router Address)");
                              break;
			    default:
                              apsr_output(" (unkown)");
	                       break;
	            }
            }
            apsr_output("\n      Length: %d\n",(int) ppp_atcp_header->ext.conf.length);
            switch(ppp_atcp_header->ext.conf.type) {
		    case APSR_PPP_ATCP_TYPE_APPLETADR:
		      apsr_output("      Net: %d, Node: %d\n", 
			apsr_ntohs(ppp_atcp_header->ext.conf.types.at_addr.at_net),
			ppp_atcp_header->ext.conf.types.at_addr.at_node);
                      break;
                    case APSR_PPP_ATCP_TYPE_ROUTING:
		      routing_proto = apsr_ntohs(ppp_atcp_header->ext.conf.types.routing.r_proto);
		      apsr_output("      Routing Protocol: %d", routing_proto);
		      if(verbose) {
			      switch(routing_proto) {
				      case APSR_PPP_ATCP_ROUTING_NONE:
					apsr_output(" (None)");
				        break;
                                      case APSR_PPP_ATCP_ROUTING_RTMP:
					apsr_output(" (RTMP)");
					break;
				      case APSR_PPP_ATCP_ROUTING_AURP:
					apsr_output(" (AURP)");
					break;
				      case APSR_PPP_ATCP_ROUTING_AGBP:
					apsr_output(" (AGBP)");
					break;
				      default:
					break;
		              }		
		      }
		      apsr_output("\n");
		      break;
		    case APSR_PPP_ATCP_TYPE_ATCOMP:
                      apsr_output("      Compression Protocol: %d\n",
                        apsr_ntohs(ppp_atcp_header->ext.conf.types.comp.c_proto)); 
		      break;
	            case APSR_PPP_ATCP_TYPE_SERVER:
                      apsr_output("      Server-Class: %d, Server-Impl-ID: %d\n",
                        apsr_ntohs(ppp_atcp_header->ext.conf.types.server.s_class),
                        apsr_ntohl(ppp_atcp_header->ext.conf.types.server.s_id)); 
		      break;
		    case APSR_PPP_ATCP_TYPE_DEFROUTER:
                      apsr_output("      Net: %d, Node: %d\n",
                        apsr_ntohs(ppp_atcp_header->ext.conf.types.at_addr.at_net),
                        ppp_atcp_header->ext.conf.types.at_addr.at_node);
		      break;
		    default:
		      break;
	    }
	    break;
	  default:
	    break;
  }	   
			
  if(statistic)
	  stats.c.ppp_atcp++;

  return(RET_SUCCESS);  
}
  
int print_ppp_lcp (unsigned char *packet, int opt_size) {
  static struct ppp_lcp_header *ppp_lcp_header;
  static unsigned char *magic_num, *seconds_rem, *message;
  static unsigned int x, length;
  
  ppp_lcp_header = (struct ppp_lcp_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_lcp_header->length);

  apsr_output("\n  - LCP Header -\n    Code: %d",(int) ppp_lcp_header->code);
  
  if(verbose) 
	  apsr_output("%s", lcpcodes[ppp_lcp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_lcp_header->identifier,(int) length);

  switch(ppp_lcp_header->code) {
	  case APSR_PPP_LCP_ECH_REQ:
          case APSR_PPP_LCP_ECH_REP:
	  case APSR_PPP_LCP_DIS_REQ:
	    apsr_output("    Magic-Num: ");
            magic_num = (packet+data_link_size+4+opt_size);
	    for(x = 0; x <= 3; x++) {
              apsr_output("%02x", magic_num[x]);
	    }
	    apsr_output("\n");			    
            break;
	  case APSR_PPP_LCP_IDENTIF:
            apsr_output("    Magic-Num: ");
            magic_num = (packet+data_link_size+4+opt_size);
            for(x = 0; x <= 3; x++) 
	              apsr_output("%02x", magic_num[x]);
            apsr_output("\n");
            break;
	    if(length > 8) {
		    message = (packet+data_link_size+4+opt_size+8);
                    apsr_output("    Message: \"");
                    for(x = 0; x <= length-8; x++) {
	              if(isprint(message[x])) 
			      apsr_output("%c", message[x]);
                      else 
                              apsr_output(".");
		    }
		    apsr_output("\"\n");
            }
	    break;
	  case APSR_PPP_LCP_TIM_REM:
	    apsr_output("    Magic-Num: ");
	    magic_num = (packet+data_link_size+4+opt_size);
	    for(x = 0; x <= 3; x++) 
              apsr_output("%02x", magic_num[x]);
            apsr_output(", Seconds-Remaining: ");
	    /* 4 octets for magic number */
	    seconds_rem = (packet+data_link_size+8+opt_size); 	
	    for(x = 0; x <= 3; x++) 
              apsr_output("%02x",  seconds_rem[x]);
	    apsr_output("\n");
            if(length > 8) {
                    message = (packet+data_link_size+12+opt_size);
                    apsr_output("    Message: \"");
                    for(x = 0; x <= length-8; x++) {
                      if(isprint(message[x])) 
                              apsr_output("%c", message[x]);
                      else 
                              apsr_output(".");
                    }
                    apsr_output("\"\n");
            }
	    break;
          case APSR_PPP_LCP_CON_REQ:
          case APSR_PPP_LCP_CON_ACK:
          case APSR_PPP_LCP_CON_NAK:
          case APSR_PPP_LCP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d",
			    (int) ppp_lcp_header->ext.conf.type);
            if(verbose) 
		    apsr_output("%s", lcpconftype[ppp_lcp_header->ext.conf.type]);

            apsr_output("\n      Length: %d\n",(int) ppp_lcp_header->ext.conf.length);
            break;
	  default:
	    break;
  }

  if(statistic) 
	  stats.c.ppp_lcp++;
  
  return(RET_SUCCESS);
}

int print_ppp_ecp (unsigned char *packet, int opt_size) {
  static struct ppp_ecp_header *ppp_ecp_header;
  
  ppp_ecp_header = (struct ppp_ecp_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - ECP Header -\n    Code: %d",(int) ppp_ecp_header->code);

  if(verbose)
	  apsr_output("%s", ext_codes[ppp_ecp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",(int) ppp_ecp_header->identifier, apsr_ntohs(ppp_ecp_header->length));
 
  switch(ppp_ecp_header->code) {
	  case APSR_PPP_ECP_CON_REQ:
	  case APSR_PPP_ECP_CON_ACK:
	  case APSR_PPP_ECP_CON_NAK:
	  case APSR_PPP_ECP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d", 
			    (int) ppp_ecp_header->ext.conf.type);
            if(verbose) {
		    switch(ppp_ecp_header->ext.conf.type) {
			    case APSR_PPP_ECP_TYPE_OUI:
                              apsr_output(" (OUI)");
                              break;
			    case APSR_PPP_ECP_TYPE_DESE:
			      apsr_output(" (DES)");
			      break;
			    case APSR_PPP_ECP_TYPE_3DESE:
			      apsr_output(" (Triple-DES)");
			      break;
			    case APSR_PPP_ECP_TYPE_DESEBIS:
			      apsr_output(" (DES Data)");
			      break;
			    default:
			      apsr_output(" (unkown)");
			      break;
                    }
            }
            apsr_output("\n      Length: %d\n",(int) ppp_ecp_header->ext.conf.length);
	    break;
	  default:
	    break;
  }
			      
  if(statistic)
	  stats.c.ppp_ecp++;

  return(RET_SUCCESS);	
}

int print_ppp_ccp (unsigned char *packet, int opt_size) {
  static struct ppp_ccp_header *ppp_ccp_header;
  
  ppp_ccp_header = (struct ppp_ccp_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - CCP Header -\n    Code: %d",(int) ppp_ccp_header->code);

  if(verbose)
	  apsr_output("%s", ext_codes[ppp_ccp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",
		  (int) ppp_ccp_header->identifier, 
		  apsr_ntohs(ppp_ccp_header->length));
 
  switch(ppp_ccp_header->code) {
	  case APSR_PPP_CCP_CON_REQ:
	  case APSR_PPP_CCP_CON_ACK:
	  case APSR_PPP_CCP_CON_NAK:
	  case APSR_PPP_CCP_CON_REJ:
            apsr_output("    - Configuration Header -\n      Type: %d", 
			    (int) ppp_ccp_header->ext.conf.type);
            if(verbose) {
		    switch(ppp_ccp_header->ext.conf.type) {
			    case APSR_PPP_CCP_TYPE_OUI:
                              apsr_output(" (OUI)");
                              break;
			    case APSR_PPP_CCP_TYPE_PRE1:
			      apsr_output(" (Predictor type 1)");
			      break;
			    case APSR_PPP_CCP_TYPE_PRE2:
			      apsr_output(" (Predictor type 2)");
			      break;
			    case APSR_PPP_CCP_TYPE_PUDDLEJUM:
			      apsr_output(" (Puddle Jumper)");
			      break;
			    case APSR_PPP_CCP_TYPE_HPPPC:
			      apsr_output(" (Hewlett-Packard PPC)");
			      break;
			    case APSR_PPP_CCP_TYPE_LZS:
			      apsr_output(" (PPP Stac LZS compression Protocol)");
			      break;
			    case APSR_PPP_CCP_TYPE_MPPC:
			      apsr_output(" (Microsoft Point-To-Point Comp. Protocol)");
			      break;
			    case APSR_PPP_CCP_TYPE_FZA:
			      apsr_output(" (PPP Gandalf FZA compression Protocol)");
			      break;
			    case APSR_PPP_CCP_TYPE_V42:
			      apsr_output(" (V.42bis compression)");
			      break;
			    case APSR_PPP_CCP_TYPE_BSD:
			      apsr_output(" (BSD compression Protocol)");
			      break;
			    case APSR_PPP_CCP_TYPE_LZS_DCP:
			      apsr_output(" (PPP LZS-DCP compression Protocol)");
			      break;
			    case APSR_PPP_CCP_TYPE_MVRCA:
			      apsr_output(" (PPP Magnalink Variable Resource compression Algorithm)");
			      break;
			    case APSR_PPP_CCP_TYPE_DCE:
			      apsr_output(" (PPP for Data Compression in Data Circuit-Terminating Equipment)");
			      break;
			    case APSR_PPP_CCP_TYPE_DEFLATE:
			      apsr_output(" (PPP Deflate Protocol)");
			      break;
			    case APSR_PPP_CCP_TYPE_V44:
			      apsr_output(" (V.44/LZJH compression Protocol)"); 
			      break;
			    default:
			      apsr_output(" (unkown)");
			      break;
                    }
            }
            apsr_output("\n      Length: %d\n",(int) ppp_ccp_header->ext.conf.length);
	    break;
	  default:
	    break;
  }
			      
  if(statistic)
	  stats.c.ppp_ccp++;

  return(RET_SUCCESS);	
}

int print_ppp_lqr (unsigned char *packet, int opt_size) {
  static struct ppp_lqr_header *ppp_lqr_header;
  
  ppp_lqr_header = (struct ppp_lqr_header *)(packet+data_link_size+opt_size);

  apsr_output("\n  - LQR Header -\n    Magic-Num: 0x%x\n",
		  ppp_lqr_header->magic_num);

  apsr_output("    LastOutLQRs: %u\n", 
		  apsr_ntohl(ppp_lqr_header->last_out_lqr));
  apsr_output("    LastOutPackets: %u\n",
                  apsr_ntohl(ppp_lqr_header->last_out_pkt));
  apsr_output("    LastOutOctets: %u\n",
                  apsr_ntohl(ppp_lqr_header->last_out_oct));
  apsr_output("    PeerInLQRs: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_in_lqr));
  apsr_output("    PeerInPackets: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_in_pkt));
  apsr_output("    PeerInDiscards: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_in_dis));
  apsr_output("    PeerInErrors: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_in_err));
  apsr_output("    PeerInOctets: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_in_oct));
  apsr_output("    PeerOutLQRs: %u\n",
	          apsr_ntohl(ppp_lqr_header->peer_out_lqr));
  apsr_output("    PeerOutPackets: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_out_pkt));
  apsr_output("    PeerOutOctets: %u\n",
                  apsr_ntohl(ppp_lqr_header->peer_out_oct));
  apsr_output("    SaveInLQRs: %u\n",
                  apsr_ntohl(ppp_lqr_header->save_in_lqr));
  apsr_output("    SaveInPackets: %u\n",
                  apsr_ntohl(ppp_lqr_header->save_in_pkt));
  apsr_output("    SaveInDiscards: %u\n",
                  apsr_ntohl(ppp_lqr_header->save_in_dis));
  apsr_output("    SaveInErrors: %u\n",
                  apsr_ntohl(ppp_lqr_header->save_in_err));
  apsr_output("    SaveInOctets: %u\n",
                  apsr_ntohl(ppp_lqr_header->save_in_oct));
  
  if(statistic)
	  stats.c.ppp_lqr++;

  return(RET_SUCCESS);	
}

int print_ppp_decnetcp (unsigned char *packet, int opt_size) {
  static struct ppp_decnetcp_header *ppp_decnetcp_header;
  static unsigned short length;

  ppp_decnetcp_header = (struct ppp_decnetcp_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_decnetcp_header->length);

  apsr_output("\n  - Decnet Phase IV CP Header -\n    Code: %d",(int) ppp_decnetcp_header->code);

  if(verbose)
          apsr_output("%s", std_codes[ppp_decnetcp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",
                  (int) ppp_decnetcp_header->identifier,(int) length);

  if(statistic)
          stats.c.ppp_decnetcp++;

  return(RET_SUCCESS);
}

int print_ppp_osinlcp (unsigned char *packet, int opt_size) {
  static struct ppp_osinlcp_header *ppp_osinlcp_header;
  static unsigned short length;

  ppp_osinlcp_header = (struct ppp_osinlcp_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_osinlcp_header->length);

  apsr_output("\n  - OSI Network Layer CP Header -\n    Code: %d",(int) ppp_osinlcp_header->code);

  if(verbose)
          apsr_output("%s", std_codes[ppp_osinlcp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",
                  (int) ppp_osinlcp_header->identifier,(int) length);

  if(statistic)
          stats.c.ppp_osinlcp++;

  return(RET_SUCCESS);
}

int print_ppp_snacp (unsigned char *packet, int opt_size) {
  static struct ppp_snacp_header *ppp_snacp_header;
  static unsigned short length;

  ppp_snacp_header = (struct ppp_snacp_header *)(packet+data_link_size+opt_size);
  length = apsr_ntohs(ppp_snacp_header->length);

  apsr_output("\n  - SNA CP Header -\n    Code: %d",(int) ppp_snacp_header->code);
 
  if(verbose)
	  apsr_output("%s", std_codes[ppp_snacp_header->code]);

  apsr_output("\n    Identifier: %d, Length: %d\n",
		  (int) ppp_snacp_header->identifier,(int) length);

  if(statistic)
	  stats.c.ppp_snacp++;

  return(RET_SUCCESS);
}


