#include "../include/aprecv-general.h"

int print_pppoe_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct pppoe_pkt_header *pppoe_header;
  static unsigned short session_id, length, version, type, code;

  pppoe_header = (struct pppoe_pkt_header *)(packet+data_link_size+opt_size);

  version 	= APSR_PPPOE_VERSION(pppoe_header);
  type		= APSR_PPPOE_TYPE(pppoe_header);
  code		= apsr_ntohs(pppoe_header->pppoe_code);
  session_id	= apsr_ntohs(pppoe_header->pppoe_session);
  length	= apsr_ntohs(pppoe_header->pppoe_length);
  
  apsr_output("\n- PPPoE Header -\n");
  if(expand_pppoe || expand_all) {
  
    apsr_output("  Version: %d, Type: %d, Code: 0x%x",(int) version,(int) type, code);

    switch(code) {
  	case APSR_PPPOE_CODE_PADI:
	  if(verbose)
	          apsr_output(" (PADI)");
	  apsr_output("\n  Session ID: 0x%x, Length: %u\n", session_id, length);
          break;
        case APSR_PPPOE_CODE_PADO:
	  if(verbose)
	          apsr_output(" (PADO)");
	  apsr_output("\n  Session ID: 0x%x, Length: %u\n", session_id, length);
          break;
        case APSR_PPPOE_CODE_PADR:
	  if(verbose)
	          apsr_output(" (PADR)");
	  apsr_output("\n  Session ID: 0x%x, Length: %u\n", session_id, length);
          break;
        case APSR_PPPOE_CODE_PADS:
	  if(verbose)
	          apsr_output(" (PADS)");
	  apsr_output("\n  Session ID: 0x%x, Length: %u\n", session_id, length);
          break;
        case APSR_PPPOE_CODE_PADT:
	  if(verbose)
	          apsr_output(" (PADT)");
	  apsr_output("\n  Session ID: 0x%x, Length: %u\n", session_id, length);
          break;
        case APSR_PPPOE_CODE_SESS:
	  if(verbose)
	          apsr_output(" (Session stage)");
	  apsr_output("\n  Session ID: 0x%x, Length: %u\n", session_id, length);
          break;
	case APSR_PPPOE_CODE_PADM:
	  if(verbose)
		  apsr_output(" (PADM)");
	  break;
	case APSR_PPPOE_CODE_PADN:
	  if(verbose)
		  apsr_output(" (PADN)");
	  break;
        default:
	  if(verbose)
		  apsr_output(" (Unknown)");
          break; 
    }
  }
    
  switch(code) {
  	case APSR_PPPOE_CODE_PADI:
        case APSR_PPPOE_CODE_PADO:
        case APSR_PPPOE_CODE_PADR:
        case APSR_PPPOE_CODE_PADS:
        case APSR_PPPOE_CODE_PADT:
	  print_pppoe_tag(packet);
          break;
        case APSR_PPPOE_CODE_SESS:
	  print_ppp_packet(header, packet, opt_size+6);
          break;
        default:
          break; 
  }
 
  if(print_payload_pppoe_hex && !print_payload_ip_hex && !print_payload_tcp_text && !print_payload_udp_text && !print_payload_tcp_hex && !print_payload_udp_hex) {
	print_raw_in_hex(packet+data_link_size+sizeof(struct pppoe_pkt_header), header->len-sizeof(struct pppoe_pkt_header)-data_link_size-1);
  }
  
  if(statistic) 
        stats.c.pppoe++;

  return(RET_SUCCESS);
}

int print_pppoe_tag (unsigned char *packet) {
  static struct pppoe_tag_header *pppoe_tag;
  static unsigned short tag_type, message_length;
  static unsigned char *message;
/*  char *temp_string;   TODO */
  unsigned short tag_length, tag_lengths=0; 
  unsigned int i=0, last_tag=2, x=0;
	
  while(last_tag != 1) {
    i++;
    pppoe_tag = (struct pppoe_tag_header *)(packet+data_link_size+6+tag_lengths);  

    tag_type = apsr_ntohs(pppoe_tag->pppoe_tag_type);
    tag_length = apsr_ntohs(pppoe_tag->pppoe_tag_length);

    apsr_output("  - Tag %d -\n    Type: 0x%x",(int) i, tag_type);
   
    switch(tag_type) {
	    case APSR_PPPOE_TAG_END:
              if(verbose)
		      apsr_output(" (End-Of-List)");
	      last_tag=1;
              break;
	    case APSR_PPPOE_TAG_SERVICE:
	      if(verbose)
		      apsr_output(" (Session Name)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_AC_NAME:
	      if(verbose)
		      apsr_output(" (AC-Name)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_HOST_UNIQ:
	      if(verbose)
		      apsr_output(" (Host-Uniq)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_AC_COOKIE:
	      if(verbose)
	      	      apsr_output(" (AC-Cookie)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_VENDOR:
	      if(verbose)
		      apsr_output(" (Vendor-Specific)");
	      message_length = tag_lengths;
              tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_RELAY_ID:
	      if(verbose)
		      apsr_output(" (Relay-Session-ID)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_SERVICE_ERR:
	      if(verbose)
		      apsr_output(" (Service-Name-Error)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_AC_ERR:
	      if(verbose)
		      apsr_output(" (AC-System-Error)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;
	    case APSR_PPPOE_TAG_GENERIC_ERR:
	      if(verbose)
		      apsr_output(" (Generic-Error)");
	      message_length = tag_lengths;
	      tag_lengths+=tag_length+4;
	      last_tag=2;
	      break;  
	    default:
	      if(verbose)
		      apsr_output(" (unknown)");
	      last_tag=1;
	      break;
    }
  
    apsr_output("\n    Length: %d\n",(int) tag_length);

    if(tag_length > 4) {
      switch(tag_type) {
	    case APSR_PPPOE_TAG_SERVICE:
	    case APSR_PPPOE_TAG_AC_NAME:
	    case APSR_PPPOE_TAG_AC_ERR:	    
	    case APSR_PPPOE_TAG_SERVICE_ERR:
	    case APSR_PPPOE_TAG_GENERIC_ERR:
	      message = (packet+data_link_size+sizeof(struct pppoe_tag_header)+6+message_length);
	      apsr_output("    Value: \"");
	      for(x = 0; x < tag_length; x++) {
		if(isprint(message[x])) 
			apsr_output("%c", message[x]);
		else 
			apsr_output(".");
	      }
	      apsr_output("\"\n");
	      break;
	    case APSR_PPPOE_TAG_RELAY_ID:
	    case APSR_PPPOE_TAG_HOST_UNIQ:
	    case APSR_PPPOE_TAG_AC_COOKIE:
/*              message = (packet+data_link_size+sizeof(struct pppoe_tag_header)+6+message_length);
	      temp_string = xmalloc(tag_length-4);

              temp_string = apsr_utf8_hunz_decode(message);
              apsr_output("    Value: \"%s\"\n", temp_string);
	      free(temp_string);*/

	      apsr_output("    Value: <UTF8 encoded, not implemented>\n"); 
	      break;
  	    default:
	      break;     
      }	
    }

    if(statistic) 
	    stats.c.pppoe_tag++;
  }
    
  return(RET_SUCCESS);
}
