#include "../include/apsrstack-general.h"

void process_udp_packet (struct apsrstack_frame_header *header, void *packet, int opt_size) {
  struct ipv4_pkt_header *ipv4_header;
  struct udp_pkt_header *udp_header;
  unsigned short c = 0, x = 0, calc_sum = 0, payload_length = 0;
  unsigned char *udp_buffer;
  
  ipv4_header = (struct ipv4_pkt_header *)(packet+data_link_size);
  udp_header = (struct udp_pkt_header *)(packet+data_link_size+opt_size);

  payload_length = apsr_ntohs(ipv4_header->packet_len) - (APSR_IPV4_HEADERLENGTH(ipv4_header)*4);    

  /*
   * Checksum verification.
   */
  memcpy(&v4_ps_hdr.ps_hdr.src_ip, &ipv4_header->src_ip.s_addr,
		  sizeof(unsigned int));
  memcpy(&v4_ps_hdr.ps_hdr.dst_ip, &ipv4_header->dst_ip.s_addr,
                  sizeof(unsigned int));
  v4_ps_hdr.ps_hdr.length = apsr_htons(apsr_ntohs(ipv4_header->packet_len)-(APSR_IPV4_HEADERLENGTH(ipv4_header)*4));
  calc_sum = apsr_udpv4_checksum(udp_header);
  if(calc_sum != 0) {
    #ifdef APSRSTACK_DEBUG
    apsr_output("DEBUG: Got UDP packet with wrong checksum.\n");
    #endif
    stats.udp.checksum_errors++; /* Checksum Errors++*/
    return;
  }

  /* 
   * Check if a apsrstack_listen is on this port 
   */
  for(c = 0; c < APSRSTACK_UDP_MAX_PORTS; c++) {
    if(udp_listen_ports[c] == udp_header->dst_port) {
      goto check_connection;
      break; /* Never reached */      
    }      
  }  
  
  /* 
   * There is no listen on this port, so we send a ICMPv4 Destination 
   * unreachable / Port unreachable packet 
   */
  payload_length = payload_length - APSR_ICMPv4;
  udp_buffer = (unsigned char *)(packet+data_link_size+opt_size);
  send_icmpv4(header,
              5,
              APSR_ICMPv4+payload_length,
              apsr_ntohs(ipv4_header->ident),
              ipv4_header->dst_ip,
              ipv4_header->src_ip,
              APSRSTACK_IPV4_DEFAULT_TTL,
              APSR_ICMP_DEST_UNREA,
              APSR_ICMP_CODE_PORT_UNREA,
	      udp_buffer,
	      payload_length);
  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: Closed port, ICMPv4 Destination unreachable / Port unreachable send.\n");
  #endif
  stats.udp.dest_un_port_un++; /* Destination Un./Port Un.++ */
  return;

check_connection:
  #ifdef APSRSTACK_DEBUG
  apsr_output("DEBUG: Port open, drop it, because its not implemented yet :-).\n");
  #endif
  stats.udp.drops++; /* Drops++ */
  return;
}

