#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="eth0";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	unsigned char oui[2] 	 = {0xcc, 0xcc};
	
	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header and 10 Bytes for the
	   802.3 LLC and SNAP */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_ETH_SNAP_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       300,		// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

	my_ret = apsr_ethernet_snap(2,		// DSAP
				    3,		// SSAP
				    4,		// Control
				    oui,	// Org ID/OUI
				    ETHERTYPE_IP,
				    buf + APSR_ETH_H);

	if(my_ret == -1) ERROR("apsr_ethernet_snap failed!");

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_ETH_SNAP_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

