#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	struct ipv4_option ipv4_opt;
	char *device="lo";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	/* 10 bytes options + 2 bytes padding */
	unsigned char *options = "123456789012";
	int my_ret, written;
	int option_s;
	unsigned char *buf;

	typedef unsigned char u_char;

	/* Set the IP options */
	option_s = strlen(options);
	memcpy(ipv4_opt.ipopt_list, options, option_s);
	*(ipv4_opt.ipopt_list)  = IPOPT_RA;
	*(ipv4_opt.ipopt_list + 1) = 1;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header, 20 bytes for the IPv4 */
	/* header and option_s bytes (40 in this case) for the IP options. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_IPv4_H + option_s, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_IP,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

 	my_ret = apsr_ipv4(4,			// IP version
			   5,			// IP header length
			   0,			// IP length
			   0x1111,		// IP Type of Service (ToS)
			   45360,		// IP Identification (ID)
			   0x4000,		// IP Fragementation offset
			   64,			// IP Time to live (TTL)
			   1,			// IP transport layer protocol
			   inet_addr("127.0.0.1"), // Source IP
			   inet_addr("127.0.0.2"), // Destination IP
			   0,			// IP checksum
			   (u_char *)&ipv4_opt,// IP options
			   option_s,		// IP option size
			   NULL,		// IP payload
			   0,			// IP payload length
			   buf + APSR_ETH_H);	// Packet buffer

	if(my_ret == -1) ERROR("apsr_ipv4 failed!");

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_IPv4_H + option_s, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

