#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"mms"
#define MODULE_DESCRIPTION	"Example MMS streaming protocol protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	1755	
#define MODULE_IDENT_ID2	1755

#define MODULE_FILTER		"tcp and port 1755"

struct mod_mms_command_header {
	unsigned char fixed[3];
	unsigned char unknown_byte;
	unsigned int command_id;
	unsigned int command_length;
	unsigned char protocol_type[4];
	unsigned int length;
	unsigned int sequence_number;
	unsigned char unknown[8];
	unsigned int length2;
	unsigned short command_value;
	unsigned short command_direction;
};

struct mod_mms_media_header {
	unsigned int sequence_number;
	unsigned char packet_id_type;
	unsigned char udp_tcp;
	unsigned short length;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_mms_command_header *mms_command_header;
  struct mod_mms_media_header *mms_media_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    mms_command_header = (struct mod_mms_command_header *)buf;
	    if(mms_command_header->fixed[0] == 0x01 &&
	       mms_command_header->fixed[1] == 0x00 &&
               mms_command_header->fixed[2] == 0x00) {
              apsr_output("- MMS Command Header -\n");
	      apsr_output("  Command ID: 0x%08x, Command Length: %u\n",
			      mms_command_header->command_id,
			      mms_command_header->command_length);
	      apsr_output("  Protocol Type: 0x%02x 0x%02x 0x%02x 0x%02x",
			      mms_command_header->protocol_type[0],
			      mms_command_header->protocol_type[1],
			      mms_command_header->protocol_type[2],
			      mms_command_header->protocol_type[3]);
	      if(verbose && mms_command_header->protocol_type[0] == 0x4D &&
                 mms_command_header->protocol_type[1] == 0x4D &&
		 mms_command_header->protocol_type[2] == 0x53 &&
		 mms_command_header->protocol_type[3] == 0x20) 
		      apsr_output(" (MMS)");
              apsr_output("\n  Length: %u, Sequence Number: %u, Length(2?): %u\n",
	  		      mms_command_header->length,
			      mms_command_header->sequence_number,
			      mms_command_header->length2);
	      apsr_output("  Command value: 0x%02x, Command direction: 0x%02x\n",
			      mms_command_header->command_value,
			      mms_command_header->command_direction);
	    }
	    else {
	      mms_media_header = (struct mod_mms_media_header *)buf;
	      apsr_output("- MMS Media Header -\n");
	      apsr_output("  Sequence Number: %u, Packet ID type: 0x%02x\n",
			      mms_media_header->sequence_number,
			      mms_media_header->packet_id_type);
	      apsr_output("  UDP Sequence / TCP flags: 0x%02x, Length: %u\n",
			      mms_media_header->udp_tcp,
			      mms_media_header->length);
	    }
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
