#include <apsrlib/apsrlib.h>

#define MODULE_VERSION        "0.02"
#define MODULE_NAME           "netbios ns"
#define MODULE_DESCRIPTION    "Example netbios ns sniffer"
#define MODULE_IDENT_FAMILY   AF_INET
#define MODULE_IDENT_PN1      APSR_PROTO_TCP
#define MODULE_IDENT_PN2      APSR_PROTO_UDP
#define MODULE_IDENT_ID1      137
#define MODULE_IDENT_ID2      137

#define MODULE_FILTER         "(tcp or udp) and port 137"


#define APSR_NETBIOS_NAMEREQUEST	0x00
#define APSR_NETBIOS_NAMEREGISTER	0x05
#define APSR_NETBIOS_NAMEFREE		0x06
#define APSR_NETBIOS_WAITRESPONSE	0x07
/* #define APSR_NAMEACTUALIZATION     0x07 */

#define APSR_NETBIOS_NB			0x0020           
#define APSR_NETBIOS_NBSTATUS		0x0021
#define APSR_NETBIOS_INET_CLASS		0x0001           

struct mod_netbios_ns_question {
  unsigned short type;
  unsigned short class;
};

struct mod_netbios_ns_flags {
  unsigned int rcode:4;
  unsigned int broadcast:1;
  unsigned int reserved2:1;
  unsigned int reserved1:1;
  unsigned int ra:1;
  unsigned int rd:1;
  unsigned int tc:1;
  unsigned int aa:1;
  unsigned int opcode:4;
  unsigned int opcode_flag:1;
};

struct mod_netbios_ns_header {
  unsigned short id;
  unsigned short code_flag;
  unsigned short request;
  unsigned short reply;
  unsigned short count;
  unsigned short additional;
};


struct apsr_aprecv_module_info module_info (void) {
  struct apsr_aprecv_module_info module_info;

  module_info.version = MODULE_VERSION;
  module_info.name = MODULE_NAME;
  module_info.desc = MODULE_DESCRIPTION;
  module_info.bpf_filter = MODULE_FILTER;
  module_info.desc = MODULE_DESCRIPTION;
  module_info.bpf_filter = MODULE_FILTER;
  module_info.family      = MODULE_IDENT_FAMILY;
  module_info.proto1      = MODULE_IDENT_PN1;
  module_info.proto2      = MODULE_IDENT_PN2;
  module_info.id1         = MODULE_IDENT_ID1;
  module_info.id2         = MODULE_IDENT_ID2;
  module_info.id3         = 0;
  module_info.id4         = 0;

  return(module_info);
}

extern char verbose;

int
module_init (unsigned char *buf, int family, unsigned short protocol_number,
             unsigned short id1, unsigned short id2)
{
	struct mod_netbios_ns_header *netbios_ns_header;
	struct mod_netbios_ns_flags *netbios_ns_flags;
	struct mod_netbios_ns_question *netbios_ns_question;
	unsigned short  blubb;
	unsigned short k,i;
	unsigned char tmp[1];
	unsigned char *buffer;

	netbios_ns_header=(struct mod_netbios_ns_header *)buf;

	blubb=netbios_ns_header->code_flag;

	tmp[0] = blubb >> 8;
        tmp[1] = blubb & 0xff;
	
	netbios_ns_flags=(struct mod_netbios_ns_flags *)tmp;

	apsr_output("- Netbios_NS Header -\n");
	apsr_output("  Transaction ID:     0x%02x\n", apsr_ntohs(netbios_ns_header->id));
	apsr_output("  Response Flag:      %u\n", netbios_ns_flags->opcode_flag);
	apsr_output("  Opcode:             %u\n", netbios_ns_flags->opcode);
	
	if(verbose) {
		apsr_output("  Flags:\n");
		apsr_output("    Answer:              %u\n", netbios_ns_flags->aa);
		apsr_output("    Truncation:          %u\n", netbios_ns_flags->tc);
		apsr_output("    Recursive desired:   %u\n", netbios_ns_flags->rd );
		apsr_output("    Recursive available: %u\n", netbios_ns_flags->ra);
		apsr_output("    Reserved1:           %u\n", netbios_ns_flags->reserved1);
		apsr_output("    Reserved2:           %u\n", netbios_ns_flags->reserved2);
		apsr_output("    Broadcast:           %u\n", netbios_ns_flags->broadcast);
	}
	
	apsr_output("  RCode:             %u\n", netbios_ns_flags->rcode);
	apsr_output("  Requests:          %u\n", apsr_ntohs(netbios_ns_header->request));
	apsr_output("  Replys:            %u\n", apsr_ntohs(netbios_ns_header->reply));
	apsr_output("  Count:             %u\n", apsr_ntohs(netbios_ns_header->count));
	apsr_output("  Additional:        %u\n", apsr_ntohs(netbios_ns_header->additional));

	buffer=(unsigned char *)(buf+sizeof(struct mod_netbios_ns_header));
	switch(netbios_ns_flags->opcode) {
		case APSR_NETBIOS_NAMEREQUEST:
		  apsr_output("  NAMEREQUEST:");
		  for(k = 0; k < apsr_ntohs(netbios_ns_header->request); k++) {
		    for(i=0; i <= 32; i++) {
		      if(i == 32 &&  apsr_ntohs(netbios_ns_header->request)-k > 1){
			apsr_output("\n  Name: ");
		      } else {
		        if(isprint(buffer[i])) 
				apsr_output("%c",buffer[i]);
			else
				apsr_output(".");
		      }
		    }
		  }
       
		  netbios_ns_question=(struct mod_netbios_ns_question *)(buffer+34);
	
		  if(verbose) {
		    switch(apsr_ntohs(netbios_ns_question->type)) {
		      case APSR_NETBIOS_NB:
			apsr_output("\n  Type: %04x (NB)", apsr_ntohs(netbios_ns_question->type));
			break;
		      case APSR_NETBIOS_NBSTATUS:
			apsr_output("\n  Type: %04x (NBStatus)", apsr_ntohs(netbios_ns_question->type));
			break; 
		      default:
			apsr_output("\n  Type: %04x (Unknown)", apsr_ntohs(netbios_ns_question->type));
			break;
		    }
		  } else
		    apsr_output("\n  Type: %04x", apsr_ntohs(netbios_ns_question->type));

		  if(verbose) {
		    switch(apsr_ntohs(netbios_ns_question->class)) {
		      case APSR_NETBIOS_INET_CLASS:
			apsr_output("\n  Class: %04x (Inetclass)\n", netbios_ns_question->class);
			break;
		      default:
			apsr_output("\n  Class: %04x (Unknown)\n", netbios_ns_question->class);
			break;
	      	    }
		  } else
			apsr_output("\n  Class: %04x\n", netbios_ns_question->class);
	
		  break;
		
		case APSR_NETBIOS_NAMEREGISTER:
		  break;
		case APSR_NETBIOS_NAMEFREE:
		  break;
		case APSR_NETBIOS_WAITRESPONSE:
		  break;
/*		case APSR_NETBIOS_NAMEACTUALIZATION:	*/
/*		  break;				*/
		default:
		  apsr_output("  Opcode(%u) not implemented.\n", netbios_ns_flags->opcode);
		  break; 
	}

	return(RET_SUCCESS);
}

