#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"rlp"
#define MODULE_DESCRIPTION	"Example RLP (Resource Location Protocol) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	39	
#define MODULE_IDENT_ID2	39

#define MODULE_FILTER		"udp and port 39"

#define APSR_RLP_TYPE_WHO		0	/* rfc 887 */
#define APSR_RLP_TYPE_DO		1       /* rfc 887 */
#define APSR_RLP_TYPE_WHO_ANY		2       /* rfc 887 */
#define APSR_RLP_TYPE_DO_ANY		3       /* rfc 887 */
#define APSR_RLP_TYPE_I			4       /* rfc 887 */
#define APSR_RLP_TYPE_THEY		5       /* rfc 887 */

static const char *mod_rlp_type[]= {
	" (Who provides?)",
	" (Do you provide?)",
	" (Who anywhere provides?)",
	" (Does anyone provide?)",
	" (I provide.)",
	" (They provide.)",
};

struct mod_rlp_header {
	unsigned char type;
	unsigned char flags;
	unsigned short message_id;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER;
        module_info.family      = MODULE_IDENT_FAMILY;
	module_info.proto1      = 0;
	module_info.proto2      = 0;
	module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_rlp_header *rlp_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    rlp_header = (struct mod_rlp_header *)buf;
	    apsr_output("- RLP Header -\n");
	    apsr_output("  Type: %u\n", rlp_header->type);
	    if(verbose) 
		    apsr_output("%s", mod_rlp_type[rlp_header->type]);
	    apsr_output("\n  Message ID: %u\n", apsr_ntohs(rlp_header->message_id));
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
