#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"sap"
#define MODULE_DESCRIPTION	"Example Session Announcement protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	9875	
#define MODULE_IDENT_ID2	9875

#define MODULE_FILTER		"tcp and port 9875"

#define SAP_VERSION(sap_ps_header)	(((sap_ps_header)->version_flags & 0xe0) >> 5)
#define SAP_FLAG_A 	0x10
#define SAP_FLAG_R	0x08
#define SAP_FLAG_T	0x04
#define SAP_FLAG_E	0x02
#define SAP_FLAG_C	0x01

struct mod_sap_header {
	unsigned char version_flags;
	unsigned char auth_length;
	unsigned short msg_id_hash;
	union {
        	struct {
			struct in_addr ip; 
		} ipv4; 
		struct {
			struct in6_addr ip;
		} ipv6;
	} source;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_sap_header *sap_header;
  unsigned short x, i=0;
  char *ip;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    sap_header = (struct mod_sap_header *)buf;
	    apsr_output("- SAP Header -\n");
	    apsr_output("  Version: %u\n", SAP_VERSION(sap_header));
	    apsr_output("  Flags: [ ");
	    if(sap_header->version_flags & SAP_FLAG_A) 
		    apsr_output("A ");
	    if(sap_header->version_flags & SAP_FLAG_R)
		    apsr_output("R ");
	    if(sap_header->version_flags & SAP_FLAG_T)
		    apsr_output("T ");
	    if(sap_header->version_flags & SAP_FLAG_E)
		    apsr_output("E ");
	    if(sap_header->version_flags & SAP_FLAG_C)
		    apsr_output("C ");
	    apsr_output("]\n  Authentication Length: %u, Message Identifier Hash: 0x%04x\n",
			    sap_header->auth_length,
			    apsr_ntohs(sap_header->msg_id_hash));

	    if(sap_header->version_flags & SAP_FLAG_A) {
		    apsr_output("  Source: ");
		    for(x = 0; x < 16; x++) {
			    if(((i++ % 2) == 0) && x != 0)
				    apsr_output(":");
			    apsr_output("%02x", 
				sap_header->source.ipv6.ip.s6_addr[x]);
		    }	    			
		    apsr_output("\n");
	    }
	    else {
 	            ip = strdup(inet_ntoa(sap_header->source.ipv4.ip));
		    apsr_output("  Source: %s\n", ip);
		    free(ip);
	    }
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
