#include <apsrlib/apsrlib.h>

#define MODULE_VERSION          "0.03"
#define MODULE_NAME             "srvloc"
#define MODULE_DESCRIPTION      "Server Location Protocol (SRVLOC/SLP) sniffer"
#define MODULE_IDENT_FAMILY     AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_PN2        APSR_PROTO_TCP
#define MODULE_IDENT_ID1	427
#define MODULE_IDENT_ID2        427

#define MODULE_FILTER		"(tcp or udp) and port 427"

/* TODO: Srvlocv2 flags		*/
/* TODO: Srvlocv2bis		*/

#define APSR_SRVLOCV1_FLAG_O	0x80
#define APSR_SRVLOCV1_FLAG_M	0x40
#define APSR_SRVLOCV1_FLAG_U	0x20
#define APSR_SRVLOCV1_FLAG_A	0x10
#define APSR_SRVLOCV1_FLAG_F	0x08

static const char *mod_srvlocv1function[]= {
	" (Unknown)",
	" (Service Request)",
	" (Service Reply)",
	" (Service Registration)",
	" (Service Deregister)",
	" (Service Acknowledge)",
	" (Attribute Request)",
	" (Attribute Reply)",
	" (DA Advertisement)",
	" (Service Type Request)",
	" (Service Type Reply)",
};

struct mod_srvlocv1_header {
	unsigned char 	version;
	unsigned char 	function;
	unsigned short 	length;
	unsigned char 	flags;
	unsigned char	dialect;
	unsigned short 	lang_code;
	unsigned short	char_enc;
	unsigned short 	xid;
};

struct mod_srvlocv2_header {
	unsigned char 	version;
	unsigned char 	function;
	unsigned char	length[2];
	unsigned short 	flags;
	unsigned char 	next_ext_offset[2];
	unsigned short 	xid;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version     = MODULE_VERSION;
	module_info.name        = MODULE_NAME;
	module_info.desc        = MODULE_DESCRIPTION;
	module_info.bpf_filter  = MODULE_FILTER;
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,                 unsigned short id1, unsigned short id2)
{
  struct mod_srvlocv1_header *srvlocv1_header;
  struct mod_srvlocv2_header *srvlocv2_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_PN1 == protocol_number || MODULE_IDENT_PN2 == protocol_number) && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	srvlocv1_header = (struct mod_srvlocv1_header *) buf;

	if(srvlocv1_header->version == 1) {
		apsr_output("- SRVLOCv1 header -\n  Version: %u", srvlocv1_header->version);
		apsr_output(", Function: %u", srvlocv1_header->function);
		if(verbose)
			apsr_output("%s", 
			  mod_srvlocv1function[srvlocv1_header->function]);
		
		apsr_output("\n  Length: %u, Flags: 0x%x", 
			apsr_ntohs(srvlocv1_header->length),
			srvlocv1_header->flags);

		if(verbose) {
			apsr_output("[ ");
			if(srvlocv1_header->flags & APSR_SRVLOCV1_FLAG_O)
				apsr_output("Overflow ");
			if(srvlocv1_header->flags & APSR_SRVLOCV1_FLAG_M)
				apsr_output("Monolingual ");
			if(srvlocv1_header->flags & APSR_SRVLOCV1_FLAG_U)
				apsr_output("URL ");
			if(srvlocv1_header->flags & APSR_SRVLOCV1_FLAG_A)
				apsr_output("Attribute Authentication Present ");
			if(srvlocv1_header->flags & APSR_SRVLOCV1_FLAG_F)
				apsr_output("Fresh ");
			apsr_output("]\n");
		}	

		apsr_output("  Dialect: %u, Language Code: %u\n",
			apsr_ntohs(srvlocv1_header->dialect),
			apsr_ntohs(srvlocv1_header->lang_code));
		apsr_output("  Char Encoding: %u, XID: %u\n",
			apsr_ntohs(srvlocv1_header->char_enc),
			apsr_ntohs(srvlocv1_header->xid));
	}
	else if(srvlocv1_header->version == 2) {
	        srvlocv2_header = (struct mod_srvlocv2_header *) buf;
		
		apsr_output("- SRVLOCv2 header -\n  Version: %u", 
				srvlocv2_header->version);

                apsr_output(", Function: %u", srvlocv2_header->function);
                if(verbose)
			apsr_output("%s",
			  mod_srvlocv1function[srvlocv2_header->function]);
	
		apsr_output("\n  Length: %u, Flags: 0x%04x",
				APSR_24TO32(srvlocv2_header->length),
				apsr_ntohs(srvlocv2_header->flags));

		apsr_output("\n  Next Extension Offset: %u, XID: %u\n",
				APSR_24TO32(srvlocv2_header->next_ext_offset),
				apsr_ntohs(srvlocv2_header->xid));
	}
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
		     family, protocol_number, id1, id2);
  #endif

  return(RET_SUCCESS);
}

