#include <apsrlib/apsrlib.h>

/* TODO: IP Address resolving */
/* TODO: Port resolving */
/* TODO: more TACACS+ support */

#define MODULE_VERSION		"0.04"
#define MODULE_NAME		"tacacs/tacacs+"
#define MODULE_DESCRIPTION	"Example TACACS/TACACS+ protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	49	
#define MODULE_IDENT_ID2	49

#define MODULE_FILTER		"(tcp or udp) and port 49"

#define APSR_TACACS_MAJOR_VERSION(tacacs_header) (((tacacs_header)->version & 0xf0) >> 4)
#define APSR_TACACS_MINOR_VERSION(tacacs_header) ((tacacs_header)->version & 0x0f)

#define APSR_TACACS_VERSION_TACACS		0x00
#define APSR_TACACS_VERSION_TACACSX		0x80

#define APSR_TACACS_TACACSX_AUTHEN		0x01	/* Authentication */
#define APSR_TACACS_TACACSX_AUTHOR		0x02	/* Authorization */
#define APSR_TACACS_TACACSX_ACCT		0x03	/* Accounting */

static const char *mod_tacacs_types[]= {
	" (Unknown)",
	" (Login)",
	" (Response)",
	" (Change)",
	" (Follow)",
	" (Connect)",
	" (Superuser)",
	" (Logout)",
	" (Reload)",
	" (Slip on)",
	" (Slip off)",
	" (Slip address)",
};

/* rfc 1492 defines */
#define APSR_TACACS_TYPE_LOGIN			1
#define APSR_TACACS_TYPE_RESPONSE		2
#define APSR_TACACS_TYPE_CHANGE			3
#define APSR_TACACS_TYPE_FOLLOW			4
#define APSR_TACACS_TYPE_CONNECT		5
#define APSR_TACACS_TYPE_SUPERUSER		6
#define APSR_TACACS_TYPE_LOGOUT			7
#define APSR_TACACS_TYPE_RELOAD			8
#define APSR_TACACS_TYPE_SLIPON			9
#define APSR_TACACS_TYPE_SLIPOFF		10
#define APSR_TACACS_TYPE_SLIPADDR		11

#define APSR_TACACS_RESPONSE_ACCEPTED		0
#define APSR_TACACS_RESPONSE_REJECTED		1

#define APSR_TACACS_REASON_ACCEPTED_NONE	0
#define APSR_TACACS_REASON_EXPIRING		1
#define APSR_TACACS_REASON_PASSWORD		2
#define APSR_TACACS_REASON_DENIED		3
#define APSR_TACACS_REASON_QUIT			4
#define APSR_TACACS_REASON_IDLE			5
#define APSR_TACACS_REASON_DROP			6
#define APSR_TACACS_REASON_BAD			7	

struct mod_tacacs_to_server_header {
	unsigned char version;
	unsigned char type;
	unsigned short nonce;
	unsigned char user_length;
	unsigned char pass_length;
	unsigned char response;
	unsigned char reason;
	unsigned int result1;
	struct in_addr dst;
	unsigned short dst_port;
	unsigned short line;
	unsigned int result2;
	unsigned short result3;
};

struct mod_tacacs_to_client_header {
	unsigned char version;
        unsigned char type;
	unsigned short nonce;
	unsigned char response;
	unsigned char reason;
};	

struct mod_tacacsplus_header {
        unsigned char version;
	unsigned char packet_type;
	unsigned char sequence_number;
	unsigned char flags;
	unsigned int session_id;
	unsigned int length;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
	module_info.proto1      = MODULE_IDENT_PN1;
	module_info.proto2      = MODULE_IDENT_PN2;
	module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_tacacs_to_client_header *tacacs_client_header;
  struct mod_tacacs_to_server_header *tacacs_server_header;
  struct mod_tacacsplus_header *tacacsplus_header;
  char *destination;
	
  tacacs_client_header = (struct mod_tacacs_to_client_header *)buf;
    /* Check for TACACS version */
  if(tacacs_client_header->version == APSR_TACACS_VERSION_TACACS) { 
      if(MODULE_IDENT_FAMILY == family && MODULE_IDENT_ID1 == id1) {
	    /* Src Port == 49 so it must be to_client */
	    tacacs_client_header = (struct mod_tacacs_to_client_header *)buf;	            
	    apsr_output("- TACACS Header -\n");
            apsr_output("  Version: %u, Type: %u",tacacs_client_header->version,
			    tacacs_client_header->type);
	    if(verbose) 
		    apsr_output("%s", mod_tacacs_types[tacacs_client_header->type]);
	    apsr_output("\n  Nonce: %u\n", 
			    apsr_ntohs(tacacs_client_header->nonce));
	    apsr_output("  Response: %u", tacacs_client_header->response);
	    if(verbose) {
		    switch(tacacs_client_header->response) {
			    case APSR_TACACS_RESPONSE_ACCEPTED:
			      apsr_output(" (accepted)");
			      break;
			    case APSR_TACACS_RESPONSE_REJECTED:
			      apsr_output(" (rejected)");
			      break;
			    default:
			      apsr_output(" (unknown)");
			      break;
		    }
	    }
	    apsr_output(", Reason: %u", tacacs_client_header->reason);
	    if(verbose) {
		    switch(tacacs_client_header->reason) {
			    case APSR_TACACS_REASON_ACCEPTED_NONE:
			      apsr_output(" (accepted/none)");
			      break;
			    case APSR_TACACS_REASON_EXPIRING:
			      apsr_output(" (expiring)");
			      break;
			    case APSR_TACACS_REASON_PASSWORD:
			      apsr_output(" (password)");
			      break;
			    case APSR_TACACS_REASON_DENIED:
			      apsr_output(" (denied)");
			      break;
			    case APSR_TACACS_REASON_QUIT:
			      apsr_output(" (quit)");
			      break;
			    case APSR_TACACS_REASON_IDLE:
			      apsr_output(" (idle)");
			      break;
			    case APSR_TACACS_REASON_DROP:
			      apsr_output(" (drop)");
			      break;
			    case APSR_TACACS_REASON_BAD:
			      apsr_output(" (bad)");
			      break;
			    default:
			      apsr_output(" (unknown)");
			      break;
		    }
            }
	    apsr_output("\n");
      }
      else if(MODULE_IDENT_FAMILY == family && MODULE_IDENT_ID1 == id2) {   
            /* Dst Port == 49 so it must be to_server */
	    tacacs_server_header = (struct mod_tacacs_to_server_header *)buf;
	    destination = strdup(inet_ntoa(tacacs_server_header->dst));
	    apsr_output("- TACACS Header -\n");
	    apsr_output("  Version: %u, Type: %u",tacacs_server_header->version,
			    tacacs_server_header->type);
            if(verbose) {
                    switch(tacacs_server_header->type) {
			    case APSR_TACACS_TYPE_LOGIN:
			      apsr_output(" (login)");
			      break;
			    case APSR_TACACS_TYPE_RESPONSE:
			      apsr_output(" (response)");
			      break;
			    case APSR_TACACS_TYPE_CHANGE:
			      apsr_output(" (change)");
			      break;
			    case APSR_TACACS_TYPE_FOLLOW:
			      apsr_output(" (follow)");
			      break;
			    case APSR_TACACS_TYPE_CONNECT:
			      apsr_output(" (connect)");
			      break;
	                    case APSR_TACACS_TYPE_SUPERUSER:
                              apsr_output(" (superuser)");
                              break;
                            case APSR_TACACS_TYPE_LOGOUT:
                              apsr_output(" (logout)");
			      break;
                            case APSR_TACACS_TYPE_RELOAD:
                              apsr_output(" (reload)");
                              break;
                            case APSR_TACACS_TYPE_SLIPON:
                              apsr_output(" (SLIP on)");
                              break;
                            case APSR_TACACS_TYPE_SLIPOFF:
                              apsr_output(" (SLIP off)");
                              break;
                            case APSR_TACACS_TYPE_SLIPADDR:
                              apsr_output(" (SLIP address)");
                              break;
                            default:
                              apsr_output(" (unknown)");
                              break;
                    }
            }
	    apsr_output("\n  Nonce: %u\n",
			    apsr_ntohs(tacacs_server_header->nonce)); 
	    apsr_output("  Username length: %u, Password length: %u\n",
			    tacacs_server_header->pass_length,
			    tacacs_server_header->user_length);
            apsr_output("  Response: %u", tacacs_server_header->response);
            if(verbose) {
		    switch(tacacs_server_header->response) {
			    case APSR_TACACS_RESPONSE_ACCEPTED:
			      apsr_output(" (accepted)");
			      break;
			    case APSR_TACACS_RESPONSE_REJECTED:
			      apsr_output(" (rejected)");
			      break;
			    default:
			      apsr_output(" (unknown)");
			      break;
		    }
            }
            apsr_output(", Reason: %u", tacacs_server_header->reason);
            if(verbose) {
		    switch(tacacs_client_header->reason) {
			    case APSR_TACACS_REASON_ACCEPTED_NONE:
			      apsr_output(" (accepted/none)");
			      break;
			    case APSR_TACACS_REASON_EXPIRING:
			     apsr_output(" (expiring)");
			     break;
			    case APSR_TACACS_REASON_PASSWORD:
		             apsr_output(" (password)");
		             break;
		            case APSR_TACACS_REASON_DENIED:
		             apsr_output(" (denied)");
			     break;
			    case APSR_TACACS_REASON_QUIT:
			     apsr_output(" (quit)");
			     break;
			    case APSR_TACACS_REASON_IDLE:
			     apsr_output(" (idle)");
			     break;
                            case APSR_TACACS_REASON_DROP:
                             apsr_output(" (drop)");
                             break;
                            case APSR_TACACS_REASON_BAD:
                              apsr_output(" (bad)");
                              break;
                            default:
                              apsr_output(" (unknown)");
                              break;
                    }
            }
	    apsr_output("  Result 1: %u\n", apsr_ntohl(tacacs_server_header->result1));
            apsr_output("  Destination Address: %s, Destination Port: %u\n",
			   destination, apsr_ntohs(tacacs_server_header->dst_port));
	    apsr_output("  Line: %u, Result 2: %u\n", 
			   tacacs_server_header->line, 
			   apsr_ntohl(tacacs_server_header->result2));
	    apsr_output("  Result 3: %u\n", 
			   apsr_ntohs(tacacs_server_header->result3));
      }
    }
    else if(tacacs_client_header->version == APSR_TACACS_VERSION_TACACSX) {
	    tacacsplus_header = (struct mod_tacacsplus_header *)buf;
	    apsr_output("- TACACS+ Header -\n");
	    apsr_output("  Major Version: %u, Minor Version: %u\n", 
			    APSR_TACACS_MAJOR_VERSION(tacacsplus_header),
	    		    APSR_TACACS_MINOR_VERSION(tacacsplus_header));
	    apsr_output("  Packet Type: %u",
			    tacacsplus_header->packet_type);
	    if(verbose) {
		    switch(tacacsplus_header->packet_type) {
			    case APSR_TACACS_TACACSX_AUTHEN:
			      apsr_output(" (Authentication)");
			      break;
			    case APSR_TACACS_TACACSX_AUTHOR:
			      apsr_output(" (Authorization)");
			      break;
			    case APSR_TACACS_TACACSX_ACCT:
			      apsr_output(" (Accounting)");
			      break;
			    default:
			      break;
		   }	      
            }
            apsr_output("\n  Sequence Number: %u, Flags: 0x%02x\n",
			    tacacsplus_header->sequence_number,
			    tacacsplus_header->flags);
	    apsr_output("  Session ID: %u, ",
			    apsr_ntohl(tacacsplus_header->session_id));
	    apsr_output(" Length: %u\n",
			    apsr_ntohl(tacacsplus_header->length));
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
